-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.0
-- //
-- //===========================================================================
-- //

import Lang.EXPR.PCKG
import Lang.EXPR.Parsers
import MechScanner.PCKG
import UU.Pretty
import SupportLib.MoreParser
import SupportLib.MoreIO
import DevTest.ParserTestLib
import Configuration.Lang



logFile :: Maybe String
logFile = Nothing
--logFile = Just "./testlogs/EXPR_parseprinting.testlog"

scanner = stringScanner scannerspec

data ParsingMode =
	  FULL
	| PL
	| QL
	deriving Show

parser mode input =  parseString scanner pExpr_ input
	where
	pExpr_ = case mode of
		FULL -> pExpr 
		PL   -> pExpr_PL 
		QL   -> pExpr_QL 

showErrorMode = ShowErrorMode_SOME 1

test mode inputs = do
	{ println ""
	; println "Testing EXPR parser in " 
	; print (show mode ++ " mode. Resulting tree is echoed:") 
	; testParser (parser mode) showErrorMode (ppExprTree False) logFile inputs 
	; println ""
        ; println "Now with pretty printing:" 
	; testParser (parser mode) showErrorMode pp logFile inputs
	; println "" 
	}
	where
	print   s = logStr logFile s
	println s = print ('\n' : s) 

runTest = do 
	{initializeLogFile logFile 
	 ; test FULL testdata1 
	 ; test PL testdata2
	 ; test QL testdata3
	}

--
-- NOTE: the parsers should be fine tuned, in particular to surpress
-- infinite repair attemps (possibly by defining the costs of various
-- constructs)
--

testdata1 =
	["1", "x", "x+y",

	 ("1 + 1 - 1 * 1 / 1 = 1 < 1 <= 1 > 1 >= 1 div 1 mod 1 " ++
	  "max 1 min 1 ==> 1 /\\ 1 \\/ 1 : #1 ++ ~1"),

	 "!i :- i range from 0 to #a -: a[i]>0",

	 "(r.Name = \"Foo Goo\") /\\ (r.Age > 100) ==> r.Type in |[\"AA\",\"AB\"]|",

	 "{ Name = \"Blob\", Job=\"Programmer\", Age=31}",

	 "select {Id = p.Id, Name = p.Name, Order = t.Order} " ++
	 "from p<-brokerDB, t<-transactionDB  " ++
	 "where (p.Id = t.Id) /\\ (t.Data = somedate)" 
	
	]

testdata2 =
	["1", "x", "x+y",

-- this causes UPL parser to go on infinite:
--	 "(r.Name = \"Foo Goo\") /\\ (r.Age > 100) ==> r.Type in |[\"AA\",\"AB\"]|",

	 "{ Name = \"Blob\", Job=\"Programmer\", Age=31}" ]

testdata3 = testdata2 ++
	["findId \"Foo\" " ++
	 "(select {Id = p.Id, Name = p.Name, Order = t.Order} " ++
	 " from p<-brokerDB, t<-transactionDB  " ++
	 " where (p.Id = t.Id) /\\ (t.Data = somedate))" 
	]



