-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //
--
-- A tool to sign files with the above header.
-- Data are read from the product info. See Configuration.
--


module SIGNATURE where

import Configuration.Face
import List
import Directory

hline1 = "-- //" ++ replicate 75 '='

hline2o = "(* //" ++ replicate 75 '='
hline2c = "-- //" ++ replicate 72 '=' ++ " *)"

authors = foo productAuthors
	where
	foo [a] = a
	foo (a:s) = foldl op a s
	op r x = r ++ ", " ++ x
--
-- Haskell & AG signature
--
signature_hs =
	[hline1,
	 "-- //",
	 "-- // AUTHOR(S): " ++ authors,
	 "-- // PACKAGE  : (c)" ++ productName
			++ ", class "  ++ productClass
			++ ", version " ++ productVersion,
	 "-- //",
	 hline1,
	 "-- //"
	]

--
-- ML & HOL signature
--
signature_ml =
	[hline2o,
	 "-- //",
	 "-- // AUTHOR(S): " ++ authors,
	 "-- // PACKAGE  : (c)" ++ productName
			++ ", class "  ++ productClass
			++ ", version " ++ productVersion,
	 "-- //",
	 hline2c,
	 "(* // *)"
	]

removeSignature content 
	=
	unlines
	. filter (not . isSigLine)
	. lines
	$ content
	where
	isSigLine row = "-- //" `isPrefixOf` row || "(* //" `isPrefixOf` row

addSignature_ signature content
	=
	unlines signature ++ content


addSignature signature fname = do
	{ putStr ("\nSigning " ++ fname ++ "...")
	; content <- readFile fname 
	; putStr . show . length $ content
	; writeFile fname . addSignature_ signature . removeSignature $ content
	}

--s `isSuffixOf` t = (reverse s) `isPrefixOf` (reverse t)

addSigAll signature dir ext = do
	{ files  <- getDirectoryContents dir ;
	  files' <- return . map (dir ++) . map ("/" ++) . filter (isSuffixOf ext) $ files ;
	  sequence . map (addSignature signature) $ files'
	} 

hsSign dir = addSigAll signature_hs dir ".hs" 
agSign dir = addSigAll signature_hs dir ".ag" 
mlSign dir = do {addSigAll signature_ml dir "Script.sml" ;
	     addSigAll signature_ml dir "Lib.sml" ;
	     addSigAll signature_ml dir ".smx" }


sign dir = do { hsSign dir ; agSign dir ; mlSign dir }

--
-- recursively sign in a directory
--


rsign dir 
	=
	do
	{ sign dir
	; files <- getDirectoryContents dir 
	; sequence . map recurse . map (\f-> dir ++ "/" ++  f) . filter g $ files
	; return ()
	}
	where

g fname = fname /= "." && fname /= ".."

recurse dir
	=
	do 
	{ isDir <- doesDirectoryExist dir
	; if isDir
	  then rsign dir
	  else return() }






