signature sdeTheory =
sig
  type thm = Thm.thm
  
  (*  Definitions  *)
    val Account : thm
    val Account_AccountNr : thm
    val Account_AccountNr_fupd : thm
    val Account_AccountNr_update : thm
    val Account_Id : thm
    val Account_Id_fupd : thm
    val Account_Id_update : thm
    val Account_Saldo : thm
    val Account_Saldo_fupd : thm
    val Account_Saldo_update : thm
    val Account_TY_DEF : thm
    val Account_case_def : thm
    val Account_repfns : thm
    val Account_size_def : thm
    val deleteById_def : thm
    val sde0_def : thm
    val sumOfPositiveSaldo_def : thm
  
  (*  Theorems  *)
    val Account_11 : thm
    val Account_AccountNr_update_semi11 : thm
    val Account_Axiom : thm
    val Account_Id_update_semi11 : thm
    val Account_Saldo_update_semi11 : thm
    val Account_accessors : thm
    val Account_accfupds : thm
    val Account_accupds : thm
    val Account_case_cong : thm
    val Account_component_equality : thm
    val Account_cupdaccs : thm
    val Account_fn_updates : thm
    val Account_induction : thm
    val Account_nchotomy : thm
    val Account_updaccs : thm
    val Account_updates : thm
    val Account_updates_eq_literal : thm
    val Account_updcanon : thm
    val Account_updupds : thm
  
  val sde_grammars : type_grammar.grammar * term_grammar.grammar
(*
   [string] Parent theory of "sde"
   
   [x_StdConst] Parent theory of "sde"
   
   [Account]  Definition
      
      |- Account = sde0
   
   [Account_AccountNr]  Definition
      
      |- !s s0 i. (Account s s0 i).AccountNr = s0
   
   [Account_AccountNr_fupd]  Definition
      
      |- !f x.
           x with AccountNr updated_by f =
           x with AccountNr := f x.AccountNr
   
   [Account_AccountNr_update]  Definition
      
      |- !s1 s s0 i. Account s s0 i with AccountNr := s1 = Account s s1 i
   
   [Account_Id]  Definition
      
      |- !s s0 i. (Account s s0 i).Id = s
   
   [Account_Id_fupd]  Definition
      
      |- !f x. x with Id updated_by f = x with Id := f x.Id
   
   [Account_Id_update]  Definition
      
      |- !s1 s s0 i. Account s s0 i with Id := s1 = Account s1 s0 i
   
   [Account_Saldo]  Definition
      
      |- !s s0 i. (Account s s0 i).Saldo = i
   
   [Account_Saldo_fupd]  Definition
      
      |- !f x. x with Saldo updated_by f = x with Saldo := f x.Saldo
   
   [Account_Saldo_update]  Definition
      
      |- !i0 s s0 i. Account s s0 i with Saldo := i0 = Account s s0 i0
   
   [Account_TY_DEF]  Definition
      
      |- ?rep.
           TYPE_DEFINITION
             (\a0'.
                !'Account'.
                  (!a0'.
                     (?a0 a1 a2.
                        a0' =
                        (\a0 a1 a2. CONSTR 0 (a0,a1,a2) (\n. BOTTOM)) a0 a1
                          a2) ==>
                     'Account' a0') ==>
                  'Account' a0') rep
   
   [Account_case_def]  Definition
      
      |- !f a0 a1 a2. case f (Account a0 a1 a2) = f a0 a1 a2
   
   [Account_repfns]  Definition
      
      |- (!a. mk_Account (dest_Account a) = a) /\
         !r.
           (\a0'.
              !'Account'.
                (!a0'.
                   (?a0 a1 a2.
                      a0' =
                      (\a0 a1 a2. CONSTR 0 (a0,a1,a2) (\n. BOTTOM)) a0 a1
                        a2) ==>
                   'Account' a0') ==>
                'Account' a0') r =
           (dest_Account (mk_Account r) = r)
   
   [Account_size_def]  Definition
      
      |- !a0 a1 a2.
           Account_size (Account a0 a1 a2) = 1 + (STRLEN a0 + STRLEN a1)
   
   [deleteById_def]  Definition
      
      |- !i db. deleteById i db = FILTER (\r. ~(r.Id = i)) db
   
   [sde0_def]  Definition
      
      |- sde0 =
         (\a0 a1 a2.
            mk_Account
              ((\a0 a1 a2. CONSTR 0 (a0,a1,a2) (\n. BOTTOM)) a0 a1 a2))
   
   [sumOfPositiveSaldo_def]  Definition
      
      |- !customer db.
           sumOfPositiveSaldo customer db =
           (SUM_ o MAP (\r. r.Saldo) o
            FILTER (\r. (r.Id = customer) /\ r.Saldo > 0)) db
   
   [Account_11]  Theorem
      
      |- !a0 a1 a2 a0' a1' a2'.
           (Account a0 a1 a2 = Account a0' a1' a2') =
           (a0 = a0') /\ (a1 = a1') /\ (a2 = a2')
   
   [Account_AccountNr_update_semi11]  Theorem
      
      |- !x y r1 r2.
           (r1 with AccountNr := x = r2 with AccountNr := y) ==> (x = y)
   
   [Account_Axiom]  Theorem
      
      |- !f. ?fn. !a0 a1 a2. fn (Account a0 a1 a2) = f a0 a1 a2
   
   [Account_Id_update_semi11]  Theorem
      
      |- !x y r1 r2. (r1 with Id := x = r2 with Id := y) ==> (x = y)
   
   [Account_Saldo_update_semi11]  Theorem
      
      |- !x y r1 r2. (r1 with Saldo := x = r2 with Saldo := y) ==> (x = y)
   
   [Account_accessors]  Theorem
      
      |- (!s s0 i. (Account s s0 i).Id = s) /\
         (!s s0 i. (Account s s0 i).AccountNr = s0) /\
         !s s0 i. (Account s s0 i).Saldo = i
   
   [Account_accfupds]  Theorem
      
      |- (!f A. (A with AccountNr updated_by f).Id = A.Id) /\
         (!f A. (A with Saldo updated_by f).Id = A.Id) /\
         (!f A. (A with Id updated_by f).AccountNr = A.AccountNr) /\
         (!f A. (A with Saldo updated_by f).AccountNr = A.AccountNr) /\
         (!f A. (A with Id updated_by f).Saldo = A.Saldo) /\
         (!f A. (A with AccountNr updated_by f).Saldo = A.Saldo) /\
         (!f A. (A with Id updated_by f).Id = f A.Id) /\
         (!f A.
            (A with AccountNr updated_by f).AccountNr = f A.AccountNr) /\
         !f A. (A with Saldo updated_by f).Saldo = f A.Saldo
   
   [Account_accupds]  Theorem
      
      |- (!x A. (A with AccountNr := x).Id = A.Id) /\
         (!x A. (A with Saldo := x).Id = A.Id) /\
         (!x A. (A with Id := x).AccountNr = A.AccountNr) /\
         (!x A. (A with Saldo := x).AccountNr = A.AccountNr) /\
         (!x A. (A with Id := x).Saldo = A.Saldo) /\
         (!x A. (A with AccountNr := x).Saldo = A.Saldo) /\
         (!x A. (A with Id := x).Id = x) /\
         (!x A. (A with AccountNr := x).AccountNr = x) /\
         !x A. (A with Saldo := x).Saldo = x
   
   [Account_case_cong]  Theorem
      
      |- !M M' f.
           (M = M') /\
           (!a0 a1 a2.
              (M' = Account a0 a1 a2) ==> (f a0 a1 a2 = f' a0 a1 a2)) ==>
           (case f M = case f' M')
   
   [Account_component_equality]  Theorem
      
      |- !A1 A2.
           (A1 = A2) =
           (A1.Id = A2.Id) /\ (A1.AccountNr = A2.AccountNr) /\
           (A1.Saldo = A2.Saldo)
   
   [Account_cupdaccs]  Theorem
      
      |- (!val A. (val = A.Id) ==> (A with Id := val = A)) /\
         (!val A. (val = A.AccountNr) ==> (A with AccountNr := val = A)) /\
         !val A. (val = A.Saldo) ==> (A with Saldo := val = A)
   
   [Account_fn_updates]  Theorem
      
      |- (!f x. x with Id updated_by f = x with Id := f x.Id) /\
         (!f x.
            x with AccountNr updated_by f =
            x with AccountNr := f x.AccountNr) /\
         !f x. x with Saldo updated_by f = x with Saldo := f x.Saldo
   
   [Account_induction]  Theorem
      
      |- !P. (!s s0 i. P (Account s s0 i)) ==> !A. P A
   
   [Account_nchotomy]  Theorem
      
      |- !A. ?s s0 i. A = Account s s0 i
   
   [Account_updaccs]  Theorem
      
      |- (!A. A with Id := A.Id = A) /\
         (!A. A with AccountNr := A.AccountNr = A) /\
         !A. A with Saldo := A.Saldo = A
   
   [Account_updates]  Theorem
      
      |- (!s1 s s0 i. Account s s0 i with Id := s1 = Account s1 s0 i) /\
         (!s1 s s0 i.
            Account s s0 i with AccountNr := s1 = Account s s1 i) /\
         !i0 s s0 i. Account s s0 i with Saldo := i0 = Account s s0 i0
   
   [Account_updates_eq_literal]  Theorem
      
      |- !s0 s i A.
           A with <|Id := s0; AccountNr := s; Saldo := i|> =
           <|Id := s0; AccountNr := s; Saldo := i|>
   
   [Account_updcanon]  Theorem
      
      |- (!z x A.
            A with <|AccountNr := x; Id := z|> =
            A with <|Id := z; AccountNr := x|>) /\
         (!z x A.
            A with <|Saldo := x; Id := z|> =
            A with <|Id := z; Saldo := x|>) /\
         !z x A.
           A with <|Saldo := x; AccountNr := z|> =
           A with <|AccountNr := z; Saldo := x|>
   
   [Account_updupds]  Theorem
      
      |- (!x2 x1 A. A with <|Id := x1; Id := x2|> = A with Id := x1) /\
         (!x2 x1 A.
            A with <|AccountNr := x1; AccountNr := x2|> =
            A with AccountNr := x1) /\
         !x2 x1 A. A with <|Saldo := x1; Saldo := x2|> = A with Saldo := x1
   
   
*)
end
