(* //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //======================================================================== *)
(* // *)
(* ============================================================
--
--
-- Authors : Ade Azurat & Wishnu Prasetya
-- Created : feb 2003
============================================================ *)

(* open Parse HolKernel boolLib bossLib  ; *)

load "integerTheory" ;
load "stringTheory" ;
load "listTheory" ;

open integerTheory stringTheory listTheory ;
  
val _ = new_theory "STMTsubstModel0" ;

(* ============================================================
-- bla bla bla kayaknya store-array lebih baik rekursif!!
-- 
-- 
============================================================ *)

val _ = Hol_datatype
	`Type =
		  TyBasicT0Int
		| TyBasicT0Bool
		| TyArray of Type`
	;

val _ = Hol_datatype
	`Literal =
		  LitInt  of int
		| LitBool of bool`
	;

val _ = Hol_datatype
	`Expr =
		  Lit	of Literal
		| Store	of Store
		| Gt 	of Expr => Expr
		| And 	of Expr => Expr
		| Plus	of Expr => Expr
		| Equal of Expr => Expr	
		| Cond	of Expr => Expr => Expr
		
	;
	Store =
		  StoreVar of string
		| ArrayApp of string => Expr` 
	;


val _ = Hol_datatype
	`Stmt =
   		  Skip
   		| Asg of Store => Expr` ;


(* a simplistic logical model of memory *)

val _ = Hol_datatype
	`CELL =	INT 		of int
		| BOOL 		of bool
	 ;
	 STORE =
		  CELL		of CELL
		| ARRAY		of CELL list
		| ERROR_STORE`
	;

val MEMORY = Type `:string -> STORE` ;

val GET_ARRAY_ELEM_def = Define 
	`GET_ARRAY_ELEM (ARRAY a) i = EL i a` ;

val UPDATE_ARRAY__def = Define
	`(UPDATE_ARRAY_ [] (i:num) x = [])
	 /\
	 (UPDATE_ARRAY_ (v::rest) 0 x = x::rest)
	 /\
	 (UPDATE_ARRAY_ (v::rest) (SUC i) x = v :: UPDATE_ARRAY_ rest i x)`
	;

val UPDATE_ARRAY_def = Define
	`(UPDATE_ARRAY (ARRAY []) i x = ERROR_STORE)
	 /\
	 (UPDATE_ARRAY (ARRAY (v::rest)) i x 
		= 
		ARRAY (UPDATE_ARRAY_ (v::rest) i x))` 
	;


val STORE2int  = Define `STORE2int  (CELL (INT i)) = i`   ;
val STORE2bool = Define `STORE2bool (CELL (BOOL b)) = b` ;
val STORE2CELL = Define `STORE2CELL (CELL c) = c` ;
 
val Gt__def = Define
	`Gt_ i j   = CELL (BOOL (STORE2int i > STORE2int j))`
	;
val And__def = Define
	`And_ p q  = CELL (BOOL (STORE2bool p /\ STORE2bool q))`
	;

val Plus__def = Define
	`Plus_ i j = CELL (INT (STORE2int i + STORE2int j))`
	;


val SEM_def = xDefine "SEM_def"
	`(Expr_SEM (Lit (LitInt i))  mem = CELL (INT i))	/\
	 (Expr_SEM (Lit (LitBool b)) mem = CELL (BOOL b))	/\
	 (Expr_SEM (Store s)   mem = Store_SEM s mem)  		/\
	 (Expr_SEM (Gt x y)    mem = Gt_   (Expr_SEM x mem) (Expr_SEM y mem)) 	/\
	 (Expr_SEM (And x y)   mem = And_  (Expr_SEM x mem) (Expr_SEM y mem)) 	/\
	 (Expr_SEM (Plus x y)  mem = Plus_ (Expr_SEM x mem) (Expr_SEM y mem)) 	/\
	 (Expr_SEM (Equal x y) mem 
		= 
		CELL (BOOL (Expr_SEM x mem = Expr_SEM y mem)))	/\
	 (Expr_SEM (Cond g x y) mem 
		=
		if STORE2bool (Expr_SEM g mem)
		then Expr_SEM x mem
		else Expr_SEM y mem	)
	  /\
	 (Store_SEM (StoreVar v) mem = mem v)
	  /\
	 (Store_SEM (ArrayApp a j) mem 
		=
		CELL (GET_ARRAY_ELEM (mem a) (Num (STORE2int (Expr_SEM j mem)))))`
	;
	
val Stmt_SEM_def = Define
	`(Stmt_SEM Skip mem = mem) 
	  /\
	 (Stmt_SEM (Asg (StoreVar x) e) mem 
		=
		(\v. if v = x then Expr_SEM e mem else mem v))
		
	  /\
	 (Stmt_SEM (Asg (ArrayApp a i) e) mem 
		=
		(\v. if v = a
		     then UPDATE_ARRAY 
				(mem a)
				(Num (STORE2int (Expr_SEM i mem)))
				(STORE2CELL (Expr_SEM e mem))
		     else mem v)
		)`
	;


val SUBST_def = xDefine "SUBST_def"
	`(SUBST (Lit c) u rhs = Lit c)
	  /\
	 (SUBST (Store s) u rhs = Store_SUBST s u rhs)
	  /\
	 (SUBST (Gt e1 e2) u rhs
		=
		Gt (SUBST e1 u rhs) (SUBST e2 u rhs))
	  /\
 	 (SUBST (And e1 e2) u rhs
		=
		And (SUBST e1 u rhs) (SUBST e2 u rhs))
	  /\
 	 (SUBST (Plus e1 e2) u rhs
		=
		Plus (SUBST e1 u rhs) (SUBST e2 u rhs))
	  /\
 	 (SUBST (Equal e1 e2) u rhs
		=
		Equal (SUBST e1 u rhs) (SUBST e2 u rhs))
	  /\
 	 (SUBST (Cond g e1 e2) u rhs
		=
		Cond (SUBST g u rhs) 
			(SUBST e1 u rhs) 
			(SUBST e2 u rhs))
	  /\
	 (Store_SUBST (StoreVar v) (StoreVar x) rhs
		=
		if v=x then rhs else Store (StoreVar v))
	  /\
	 (Store_SUBST (ArrayApp a i) (StoreVar x) (Store (StoreVar z))
		=
		let
		i_ = SUBST i (StoreVar x) (Store (StoreVar z))
		in
		if a=x 
		then Store (ArrayApp z i_ )
		else Store (ArrayApp a i_ ))
	  /\
	 (Store_SUBST (ArrayApp a i) (ArrayApp b j) rhs
		=
		let
		i_ = SUBST i (ArrayApp b j) rhs
		in
		if a = b
		then Cond (Equal i_ j) rhs (Store (ArrayApp a i_ ))
		else Store (ArrayApp a i_ ))`
	;

val Get_Array_Elem_ty = Define `Get_Array_Elem_ty (TyArray ty) = ty` ;

val Is_Array_Type_def     = Define 
	`(Is_Array_Type TyBasicT0Int = F) 	/\
	 (Is_Array_Type (TyArray ty) = T)	/\
	 (Is_Array_Type TyBasicT0Bool = F)`
	;

val TYPE_OF_def = xDefine "TYPE_OF_def"
	`(TYPE_OF_Lit (LitInt i)  = TyBasicT0Int)	/\
	 (TYPE_OF_Lit (LitBool b) = TyBasicT0Bool)	/\

	 (TYPE_OF_Expr (Lit c)      env = TYPE_OF_Lit c)	/\
	 (TYPE_OF_Expr (Store s)    env = TYPE_OF_Store s env)	/\
	 (TYPE_OF_Expr (Gt x y)     env = TyBasicT0Bool)	/\
	 (TYPE_OF_Expr (And x y)    env = TyBasicT0Bool)	/\
	 (TYPE_OF_Expr (Plus x y)   env = TyBasicT0Int)		/\
	 (TYPE_OF_Expr (Equal x y)  env = TyBasicT0Bool)	/\
	 (TYPE_OF_Expr (Cond g x y) env = TYPE_OF_Expr x env)	/\

	 (TYPE_OF_Store (StoreVar v)   env = env v)		/\
	 (TYPE_OF_Store (ArrayApp a j) env = Get_Array_Elem_ty (env a))`
	;

val TYPE_CORRECT_def = xDefine "TYPE_CORRECT_def"
	`(TYPE_CORRECT_Expr (Lit c)      env = T)		/\
	 (TYPE_CORRECT_Expr (Store s)    env = TYPE_CORRECT_Store s env)	/\
	 (TYPE_CORRECT_Expr (Gt x y)     env 
		=
		TYPE_CORRECT_Expr x env /\
		TYPE_CORRECT_Expr y env /\
		(TYPE_OF_Expr x env  = TyBasicT0Int) /\
		(TYPE_OF_Expr y env  = TyBasicT0Int))		/\
	 (TYPE_CORRECT_Expr (And x y)     env 
		=
		TYPE_CORRECT_Expr x env /\
		TYPE_CORRECT_Expr y env /\
		(TYPE_OF_Expr x env  = TyBasicT0Bool) /\
		(TYPE_OF_Expr y env  = TyBasicT0Bool))		/\
	 (TYPE_CORRECT_Expr (Plus x y)     env 
		=
		TYPE_CORRECT_Expr x env /\
		TYPE_CORRECT_Expr y env /\
		(TYPE_OF_Expr x env  = TyBasicT0Int) /\
		(TYPE_OF_Expr y env  = TyBasicT0Int))		/\
	 (TYPE_CORRECT_Expr (Equal x y)     env 
		=
		TYPE_CORRECT_Expr x env /\
		TYPE_CORRECT_Expr y env /\
		(TYPE_OF_Expr x env  = TYPE_OF_Expr y env))	/\
	 (TYPE_CORRECT_Expr (Cond g x y) env 
		= 
		TYPE_CORRECT_Expr g env /\
		TYPE_CORRECT_Expr x env /\
		TYPE_CORRECT_Expr y env /\
		(TYPE_OF_Expr g env  = TyBasicT0Bool) /\
		(TYPE_OF_Expr x env  = TYPE_OF_Expr y env))	/\

	 (TYPE_CORRECT_Store (StoreVar v)  env = T)	/\
	 (TYPE_CORRECT_Store (ArrayApp a j) env 
		= 
		TYPE_CORRECT_Expr j env			/\
		(TYPE_OF_Expr j env = TyBasicT0Int)	/\
		(Is_Array_Type (env a)))			/\


	 (TYPE_CORRECT_Stmt Skip env = T)			/\
	 (TYPE_CORRECT_Stmt (Asg (StoreVar v) e) env
		=
		TYPE_CORRECT_Expr e env		/\
		(env v = TYPE_OF_Expr e env))			/\

	 (TYPE_CORRECT_Stmt (Asg (ArrayApp a j) e) env
		=
		TYPE_CORRECT_Store (ArrayApp a j) env		/\
		TYPE_CORRECT_Expr e env		/\
		(TYPE_OF_Expr j env = TyBasicT0Int) /\
		(Get_Array_Elem_ty (env a) = TYPE_OF_Expr e env))`
	;


(* Forbid cond-expr at the rhs of asg as it requires more 
sophisticated representation of store *)

val SYNTAX_CORRECT_Stmt_def = Define 
	`(SYNTAX_CORRECT_Stmt Skip env = T)			
	  /\
	 (SYNTAX_CORRECT_Stmt (Asg (ArrayApp a i) e) env  = T)
	  /\	
	 (SYNTAX_CORRECT_Stmt (Asg target (Lit c)) env = T)
	  /\
	 (SYNTAX_CORRECT_Stmt (Asg target (Store s)) env = T)
	  /\
	 (SYNTAX_CORRECT_Stmt (Asg target (Gt e1 e2)) env = T)
	  /\
	 (SYNTAX_CORRECT_Stmt (Asg target (Plus e1 e2)) env = T)
	  /\
	 (SYNTAX_CORRECT_Stmt (Asg target (Equal e1 e2)) env = T)
	  /\
	 (SYNTAX_CORRECT_Stmt (Asg target (Cond g e1 e2)) env
		=
		~ (Is_Array_Type (env v)))`
	;

(* forbid individual store-var x of type array at the post-condition
as it requires introduction of ArrayUpdate construct in Expr, which in
turns will require a more sophisticated representation of store *)

val SYNTAX_CORRECT_PostC_def = Define 
	`(SYNTAX_CORRECT_PostC (Lit c) env = T)
	  /\
	 (SYNTAX_CORRECT_PostC (Store (StoreVar v)) env 
		= 
		~(Is_Array_Type (env v)))
	  /\
	 (SYNTAX_CORRECT_PostC (Store (ArrayApp a i)) env 
		= 
		SYNTAX_CORRECT_PostC i env )
	  /\
	 (SYNTAX_CORRECT_PostC (Gt e1 e2) env 
		= 
		SYNTAX_CORRECT_PostC e1 env /\ SYNTAX_CORRECT_PostC e2 env)
	  /\
	 (SYNTAX_CORRECT_PostC (And e1 e2) env 
		= 
		SYNTAX_CORRECT_PostC e1 env /\ SYNTAX_CORRECT_PostC e2 env)
	  /\
	 (SYNTAX_CORRECT_PostC (Plus e1 e2) env  
		= 
		SYNTAX_CORRECT_PostC e1 env /\ SYNTAX_CORRECT_PostC e2 env)
	  /\
	 (SYNTAX_CORRECT_PostC (Equal e1 e2) env
		= 
		SYNTAX_CORRECT_PostC e1 env /\ SYNTAX_CORRECT_PostC e2 env)
	  /\
	 (SYNTAX_CORRECT_PostC (Cond g e1 e2) env 
		= 
		SYNTAX_CORRECT_PostC g env  /\ 
		SYNTAX_CORRECT_PostC e1 env /\
		SYNTAX_CORRECT_PostC e2 env)`
	;

xxx;


val HOA_def = Define `HOA stmt p q = !s. p s ==> q (stmt s)` ;


prove this::

(1)
	--`!target q e.
		TYPE_CORRECT_Expr q type_env 			/\
		TYPE_CORRECT_Stmt (Asg target e) type_env	/\
		SYNTAX_CORRECT_Stmt (Asg target e) 		/\
		SYNTAX_CORRECT_PostC q type_env
		==>
		(Expr_SEM q o Stmt_SEM (Asg target e)
		 =
		 Expr_SEM (SUBST q target e))`-- 

	(CONV_TAC o DEPTH_CONV) FUN_EQ_CONV
	THEN REWRITE_TAC [combinTheory.o_DEF]
	THEN BETA_TAC
	THEN Induct
	THENL

	[ (* target is just an identifier *)
	  GEN_TAC
	  THEN Induct
	  THEN TRY (* this is to solve the cases :
		wp (x:=e) {d1 > d2} 
	    	wp (x:=e) {d1 /\ d2}
	    	wp (x:=e) {d1 + d2} 
	    	wp (x:=e) {d1 = d2} 
	    	wp (x:=e) {if g then d1 else d2} *)
	    	(RW_TAC std_ss [SEM_def, SUBST_def, TYPE_CORRECT_def, 
			SYNTAX_CORRECT_PostC_def] 
		THEN NO_TAC)
	  THEN TRY 
		(* this is for the case: wp (x:=e) {constant} *)
	    	(Induct
	    	 THEN RW_TAC std_ss [SUBST_def, SEM_def]
	    	 THEN NO_TAC)

	xxx last ...

	  THEN (* the remaining case:  wp (x:=e) {store} *)
	    	SPEC_TAC (--`S':Store`--, --`targetStore:Store`--)
    	  THEN Induct
	  THEN TRY 
		(* case: wp (x:=e) {y} --y is just an ident*)
		(RW_TAC std_ss [SEM_def, SUBST_def, Stmt_SEM_def] THEN NO_TAC)
	  THEN (* case: wp (s:=e) {s'[E]} *)


	  THEN SPEC_TAC (--`E:Expr`--, --`E:Expr`--)
    	  THEN Induct

	  THEN GEN_TAC 
	  THEN ASM_CASES_TAC (--`(s:string) = s'`--)
	  THEN Induct
	  THEN TRY (RW_TAC std_ss [SEM_def, SUBST_def, TYPE_CORRECT_def] THEN NO_TAC)


	,
	]

	) ;



(2)

HOA (Stmt_SEM (Asg (ArrayApp a i) e)) 
	(Expr_SEM (SUBST q (getframe a) (ArrayUpdate (getframe a) (getIndices a i) e))
	(Expr_SEM q)






	
val _ = export_theory() ; 
