(* //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //======================================================================== *)
(* // *)
(* ============================================================
--
--
-- Authors : Ade Azurat & Wishnu Prasetya
-- Created : feb 2003
============================================================ *)

(* open Parse HolKernel boolLib bossLib  ; *)

load "integerTheory" ;
load "stringTheory" ;
load "listTheory" ;

open integerTheory stringTheory listTheory ;
  
val _ = new_theory "STMTsubstModel1" ;

(* ============================================================
--
-- 
-- 
============================================================ *)


(* a logical model of memory *)

val SYMBOLIC_LOCAL_ADDRESS = Type `:string` ;

val _ = Hol_datatype
	`CELL =	INT 		of int
		| BOOL 		of bool
	 ;
	 STORE =
		  CELL		of CELL
		| BLOCK		of (^SYMBOLIC_LOCAL_ADDRESS # STORE) list`
	;

val _ = Hol_datatype
	`LOCAL_ADDRESS =
		  SYMBOLIC of ^SYMBOLIC_LOCAL_ADDRESS
		| PHYSICAL of int`
	;

val ADDRESS = Type `:LOCAL_ADDRESS list` ;

val EL__def = Define
	`EL_ i (x::s) 
		=  
		if i=0 then SND x else EL_ (i-1) s` ;

val LOOKUP_def = Define
	`LOOKUP name (x::s)
		=
		if name = FST x 
		then SND x 
		else LOOKUP name s` ;

val READ1_def = Define
	`(READ1 (SYMBOLIC x) (BLOCK a) = LOOKUP x a)	/\
	 (READ1 (PHYSICAL i) (BLOCK a) = EL_ i a)	`
	;
	 
val READ_def = Define
	`(READ [] store        = store)	/\
	 (READ (a::rest) store = READ rest (READ1 a store))` ;

val UPDATE_def = Define
	`(UPDATE i newval []  = [])	/\
	 (UPDATE i newval (x::s) 
		=
		if i=0
		then (FST x, newval) :: s
		else x :: UPDATE (i - 1) newval s)` ;

val UPDATE_LOOKUP_def = Define
	`(UPDATE_LOOKUP name newval [] = [])	/\
	 (UPDATE_LOOKUP name newval (x::s) 
		=
		if FST x = name
		then (FST x, newval) :: s
		else x :: UPDATE_LOOKUP name newval s)` ;

val WRITE1_def = Define
	`(WRITE1 (SYMBOLIC x) (BLOCK a) newval 
		= 
		BLOCK (UPDATE_LOOKUP x newval a))	
	  /\
	 (WRITE1 (PHYSICAL i) (BLOCK a) newval 
		= 
		BLOCK (UPDATE i newval a))`
	;
	 
val WRITE_def = Define
	`(WRITE [] store  newval       = store)	/\
	 (WRITE (a::rest) store newval 
		= 
		WRITE1 a store (WRITE rest (READ1 a store) newval))`
	;


val _ = Hol_datatype
	`Type =
		  TyBasicT0Int
		| TyBasicT0Bool
		| TyArray of Type`
	;

val _ = Hol_datatype
	`Literal =
		  LitInt  of int
		| LitBool of bool`
	;

val _ = Hol_datatype
	`Expr =
		  Lit	of Literal
		| Ident of string
		| ArrayApp of Expr => Expr list
		| ArrayUpdate of Expr => Expr list => Expr
		| Gt 	of Expr => Expr
		| And 	of Expr => Expr
		| Plus	of Expr => Expr
		| Equal of Expr => Expr	
		| Cond	of Expr => Expr => Expr` ;

val _ = Hol_datatype
	`Stmt =
   		  Skip
   		| Asg of Expr => Expr` ;

val _ = new_constant ("Subst" , Type `:Expr->Expr->Expr->Expr`) ;

val Subst_def_ax = mk_oracle_thm (Tag.read "Subst_def_ax") 
	([],
	--`(Subst (Lit c) target rhs  = Lit c)
	  /\
	 (Subst (Ident  x) (Ident y) rhs 
		=
		if x=y then rhs else Ident x)
	  /\
	 (Subst (Ident x) (ArrayApp (Ident t) indices) rhs 
		=
		if x = t
		then ArrayUpdate (Ident x) indices rhs
		else Ident x)
	  /\
	 (Subst (ArrayApp a indices) target rhs 
		=
		ArrayApp (Subst a target rhs) 
			 (MAP (\i. Subst i target rhs) indices))
	  /\
	 (Subst (ArrayUpdate a indices newval) target rhs 
		=
		ArrayUpdate 
			(Subst a target rhs)
			(MAP (\i. Subst i target rhs) indices)
			(Subst newval target rhs)) 
	  /\
	 (Subst (Gt e1 e2) target rhs 
		=
		Gt (Subst e1 target rhs) (Subst e2 target rhs))
	  /\
 	 (Substt (And e1 e2) target rhs 
		=
		And (Subst e1 target rhs) (Subst e2 target rhs))
	  /\
 	 (Subst (Plus e1 e2) target rhs 
		=
		Plus (Subst e1 target rhs) (Subst e2 target rhs))
	  /\
 	 (Subst (Equal e1 e2) target rhs 
		=
		Equal (Subst e1 target rhs) (Subst e2 target rhs))
	  /\
 	 (Subst (Cond g e1 e2) target rhs 
		=
		Cond (Subst g target rhs) 
			(Subst e1 target rhs) 
			(Subst e2 target rhs))
	`--)
	;


val STORE2int  = Define `STORE2int  (CELL (INT i)) = i`   ;
val STORE2bool = Define `STORE2bool (CELL (BOOL b)) = b` ;
 
val Gt__def = Define
	`Gt_ i j   = CELL (BOOL (STORE2int i > STORE2int j))`
	;
val And__def = Define
	`And_ p q  = CELL (BOOL (STORE2bool p /\ STORE2bool q))`
	;

val Plus__def = Define
	`Plus_ i j = CELL (INT (STORE2int i + STORE2int j))`
	;

val _ = new_constant ("Expr_SEM", Type `:STORE->Expr->STORE`) ;

val Expr_SEM_def_ax = mk_oracle_thm (Tag.read "Expr_SEM_def_ax") 
	([],
	--`(Expr_SEM mem (Lit (LitInt i))  = CELL (INT i))		/\
	 (Expr_SEM mem (Lit (LitBool b)) = CELL (BOOL b))		/\
	 (Expr_SEM mem (Ident v)          = READ [SYMBOLIC v] mem) 	/\
	 (Expr_SEM mem (ArrayApp a indices) 
		= 
		READ (MAP (PHYSICAL o STORE2int o Expr_SEM mem) indices)
		     (Expr_SEM mem a))
	  /\
	 (Expr_SEM mem (ArrayUpdate a indices newval) 
		= 
		WRITE 
		  (MAP (PHYSICAL o STORE2int o Expr_SEM mem) indices)
		  (Expr_SEM mem a)
		  (Expr_SEM mem newval))
	  /\
	 (Expr_SEM mem (Gt x y)    = Gt_   (Expr_SEM mem x) (Expr_SEM mem y)) 	/\
	 (Expr_SEM mem (And x y)   = And_  (Expr_SEM mem x) (Expr_SEM mem y)) 	/\
	 (Expr_SEM mem (Plus x y)  = Plus_ (Expr_SEM mem x) (Expr_SEM mem y)) 	/\
	 (Expr_SEM mem (Equal x y)  
		= 
		CELL (BOOL (Expr_SEM mem x = Expr_SEM mem y)))	/\
	 (Expr_SEM mem (Cond g x y) 
		=
		if STORE2bool (Expr_SEM mem g)
		then Expr_SEM mem x
		else Expr_SEM mem y	)
	`--)
	;
	
val Stmt_SEM_def = Define
	`(Stmt_SEM Skip mem = mem) 
	  /\
	 (Stmt_SEM (Asg (Ident x) e) mem
		=
		WRITE [SYMBOLIC x] mem (Expr_SEM mem e))
	  /\
	 (Stmt_SEM (Asg (ArrayApp (Ident a) indices) e) mem 
		=
		WRITE 
		  (SYMBOLIC a :: MAP (PHYSICAL o STORE2int o Expr_SEM mem) indices)
		   mem
		   (Expr_SEM mem e))` ;


val Get_Array_Elem_ty = Define `Get_Array_Elem_ty (TyArray ty) = ty` ;

val TYPE_OF_def = xDefine "TYPE_OF_def"
	`(TYPE_OF_Lit (LitInt i)  = TyBasicT0Int)	/\
	 (TYPE_OF_Lit (LitBool b) = TyBasicT0Bool)	/\

	 (TYPE_OF_Expr (Lit c)      env = TYPE_OF_Lit c)	/\
	 (TYPE_OF_Expr (Ident v)    env = env v)		/\
 	 (TYPE_OF_Expr (ArrayApp a indices) env 
		= 
		Get_Array_Elem_ty (TYPE_OF_Expr a env))		/\
 	 (TYPE_OF_Expr (ArrayUpdate a indices newval) env 
		= 
		TYPE_OF_Expr a env)				/\
	 (TYPE_OF_Expr (Gt x y)     env = TyBasicT0Bool)	/\
	 (TYPE_OF_Expr (And x y)    env = TyBasicT0Bool)	/\
	 (TYPE_OF_Expr (Plus x y)   env = TyBasicT0Int)		/\
	 (TYPE_OF_Expr (Equal x y)  env = TyBasicT0Bool)	/\
	 (TYPE_OF_Expr (Cond g x y) env = TYPE_OF_Expr x env)	
	`
	;

val Is_Array_Type_def     = Define 
	`(Is_Array_Type TyBasicT0Int = F) 	/\
	 (Is_Array_Type (TyArray ty) = T)	/\
	 (Is_Array_Type TyBasicT0Bool = F)`
	;


val _ = new_constant ("TYPE_CORRECT_Expr", Type `:Expr->(string->Type)->bool`) ;

val TYPE_CORRECT_Expr_def_ax = mk_oracle_thm (Tag.read "TYPE_CORRECT_Expr_def_ax") 
	([],
	--`(TYPE_CORRECT_Expr (Lit c)      env = T)		/\

	 (TYPE_CORRECT_Expr (Ident v)  env = T)			/\

	 (TYPE_CORRECT_Expr (ArrayApp a indices) env 
		= 
		TYPE_CORRECT_Expr a env				/\
		EVERY (\j. TYPE_CORRECT_Expr j env) indices	/\
		EVERY (\j. TYPE_OF_Expr j env = TyBasicT0Int) indices	/\
		(Is_Array_Type (TYPE_OF_Expr a env)))		/\

	 (TYPE_CORRECT_Expr (ArrayUpdate a indices newval) env 
		= 
		TYPE_CORRECT_Expr a env				/\
		EVERY (\j. TYPE_CORRECT_Expr j env) indices	/\
		TYPE_CORRECT_Expr newval env				/\
		EVERY (\j. TYPE_OF_Expr j env = TyBasicT0Int) indices	/\
		(Is_Array_Type (TYPE_OF_Expr a env))		/\
		(TYPE_OF_Expr newval env 
			= 
			Get_Array_Elem_ty (TYPE_OF_Expr newval env)))	/\

	 (TYPE_CORRECT_Expr (Gt x y)     env 
		=
		TYPE_CORRECT_Expr x env /\
		TYPE_CORRECT_Expr y env /\
		(TYPE_OF_Expr x env  = TyBasicT0Int) /\
		(TYPE_OF_Expr y env  = TyBasicT0Int))		/\

	 (TYPE_CORRECT_Expr (And x y)     env 
		=
		TYPE_CORRECT_Expr x env /\
		TYPE_CORRECT_Expr y env /\
		(TYPE_OF_Expr x env  = TyBasicT0Bool) /\
		(TYPE_OF_Expr y env  = TyBasicT0Bool))		/\

	 (TYPE_CORRECT_Expr (Plus x y)     env 
		=
		TYPE_CORRECT_Expr x env /\
		TYPE_CORRECT_Expr y env /\
		(TYPE_OF_Expr x env  = TyBasicT0Int) /\
		(TYPE_OF_Expr y env  = TyBasicT0Int))		/\

	 (TYPE_CORRECT_Expr (Equal x y)     env 
		=
		TYPE_CORRECT_Expr x env /\
		TYPE_CORRECT_Expr y env /\
		(TYPE_OF_Expr x env  = TYPE_OF_Expr y env))	/\

	 (TYPE_CORRECT_Expr (Cond g x y) env 
		= 
		TYPE_CORRECT_Expr g env /\
		TYPE_CORRECT_Expr x env /\
		TYPE_CORRECT_Expr y env /\
		(TYPE_OF_Expr g env  = TyBasicT0Bool) /\
		(TYPE_OF_Expr x env  = TYPE_OF_Expr y env))
	`--) ;

val TYPE_CORRECT_Stmt_def = Define
	`(TYPE_CORRECT_Stmt Skip env = T)
	  /\
	 (TYPE_CORRECT_Stmt (Asg (Lit c) e) env = F)		
	  /\
	 (TYPE_CORRECT_Stmt (Asg (Ident v) e) env
		=
		TYPE_CORRECT_Expr e env		/\
		(env v = TYPE_OF_Expr e env))
	  /\
	 (TYPE_CORRECT_Stmt (Asg (ArrayApp a indices) e) env
		=
		TYPE_CORRECT_Expr (ArrayApp a indices) env	/\
		TYPE_CORRECT_Expr e env				/\
		(Get_Array_Elem_ty (TYPE_OF_Expr a env) = TYPE_OF_Expr e env))
	  /\
	 (TYPE_CORRECT_Stmt (Asg (ArrayUpdate a indices newval) e) env = F)
	  /\
	 (TYPE_CORRECT_Stmt (Asg (Gt x y) e) env = F)
	  /\
	 (TYPE_CORRECT_Stmt (Asg (And x y) e) env = F)
	  /\
	 (TYPE_CORRECT_Stmt (Asg (Plus x y) e) env = F)
	  /\
	 (TYPE_CORRECT_Stmt (Asg (Equal x y) e) env = F)
	  /\
	 (TYPE_CORRECT_Stmt (Asg (Cond g x y) e) env = F)
	`
	;


val HOA_def = Define `HOA stmt p q = !s. p s ==> q (stmt s)` ;


prove this::

(1)
	--`!q e.
		TYPE_CORRECT_Expr q type_env 			/\
		TYPE_CORRECT_Stmt (Asg (Ident v) e) type_env	
		==>
		(!state. 
			Expr_SEM (Stmt_SEM (Asg (Ident v) e) state) q
		 	=
		 	Expr_SEM state (Subst q (Ident v) e))
	`-- 

	Induct
	THEN TRY (* this is to solve the cases :
		wp (x:=e) {d1 > d2} 
	    	wp (x:=e) {d1 /\ d2}
	    	wp (x:=e) {d1 + d2} 
	    	wp (x:=e) {d1 = d2} 
	    	wp (x:=e) {if g then d1 else d2} *)
	    	(RW_TAC std_ss [Expr_SEM_def_ax, 
				Stmt_SEM_def,
				Subst_def_ax, 
				TYPE_CORRECT_Expr_def_ax,
				TYPE_CORRECT_Stmt_def ]
		 THEN NO_TAC)
	  THEN TRY 
		(* this is for the case: wp (x:=e) {constant} *)
	    	(Induct
	    	 THEN RW_TAC std_ss [Subst_def_ax, Expr_SEM_def_ax, Stmt_SEM_def]
	    	 THEN NO_TAC)

	xxx last ...

	  THEN (* the remaining case:  wp (x:=e) {store} *)
	    	SPEC_TAC (--`S':Store`--, --`targetStore:Store`--)
    	  THEN Induct
	  THEN TRY 
		(* case: wp (x:=e) {y} --y is just an ident*)
		(RW_TAC std_ss [SEM_def, SUBST_def, Stmt_SEM_def] THEN NO_TAC)
	  THEN (* case: wp (s:=e) {s'[E]} *)


	  THEN SPEC_TAC (--`E:Expr`--, --`E:Expr`--)
    	  THEN Induct

	  THEN GEN_TAC 
	  THEN ASM_CASES_TAC (--`(s:string) = s'`--)
	  THEN Induct
	  THEN TRY (RW_TAC std_ss [SEM_def, SUBST_def, TYPE_CORRECT_def] THEN NO_TAC)


	,
	]

	) ;



(2)

HOA (Stmt_SEM (Asg (ArrayApp a i) e)) 
	(Expr_SEM (SUBST q (getframe a) (ArrayUpdate (getframe a) (getIndices a i) e))
	(Expr_SEM q)






	
val _ = export_theory() ; 
