(* //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //======================================================================== *)
(* // *)
(* ============================================================
--
--
-- Authors : Ade Azurat & Wishnu Prasetya
-- Created : feb 2003
============================================================ *)

open Parse HolKernel boolLib bossLib  ;

(*

load "integerTheory" ;
load "stringTheory" ;
load "listTheory" ;

open integerTheory stringTheory listTheory ;
  
val _ = new_theory "MechDataTypeScript" ;

(* ============================================================
--
-- 
-- 
============================================================ *)


val _ = Hol_datatype
	`BasicType = T0Int | T0Bool` ;

val _ = Hol_datatype
	`Type =  
		  TyFunc 	of Type => Type	
		| TyArray	of Type     
		| TyForall	of string list => Type
		| TyAddress	of Type
		| TyVar  	of string
		| TyBasic	of BasicType
		| TyUnknown` 
	;

val _ = Hol_datatype
	`IdentStatus =
		  LogicConst	
		| LogicVar	
		| BoundVar	
		| ProgramVar
		| AuxProgramVar
		| UnknownIdentStatus`
	;

val _ = Hol_datatype
	`Ident = Identifier of 
			string =>
			IdentStatus =>
			Type`
	;

val _ =  Hol_datatype
	`IdentList =
		  IdentNil
		| IdentCons of Ident => IdentList`
	;

val _ = Hol_datatype
	`Literal =
		  LitInt  of int
		| LitBool of bool`
	;

val _ = Hol_datatype
	`Expr =
		  Lit	of Literal
		| Store	of Store
		| Gt 	of Expr => Expr
		| And 	of Expr => Expr
		| Plus	of Expr => Expr
	;
	ExprList =
		  ExprNil
		| ExprCons of Expr => ExprList
	;
	Store =
		  StoreVar of Ident
		| ArrayApp of Store => Expr
		| ArrayUpdate of	
			Store =>
			ExprList =>
			Expr	
	` ;


val _ = Hol_datatype
	`Stmt =
   		  Skip
   		| Asg of Store => Expr` ;


(* logical model of memory *)

val _ = Hol_datatype
	`STORE =
		  INT 		of int
		| BOOL 		of bool
		| ARRAY		of STORE list
		| ERRORVALUE`
	;

val MEMORY = Type `:string -> STORE` ;

val GET_ARRAY_ELEM_def = Define 
	`GET_ARRAY_ELEM i (ARRAY a) = EL i a` ;

val _ = new_constant (
		"UPDATE_ARRAY", 
		Type `:STORE -> num list -> STORE -> STORE`) ;

val _ = new_constant (
		"UPDATE_ARRAY_", 
		Type `:STORE list -> num list -> STORE -> STORE list`) ;

val UPDATE_ARRAY_ora1 =  
	mk_oracle_thm 
	  (Tag.read "UPDATE_ARRAY_ora1")
	  ([],--`(UPDATE_ARRAY s [] newval = newval)
		  /\
		 (UPDATE_ARRAY (ARRAY a) (i::rest) newval
			=
			ARRAY (UPDATE_ARRAY_ a (i::rest) newval))
		  /\
		 (UPDATE_ARRAY_ (x::xs) ((0:num)::rest) newval
			=
			UPDATE_ARRAY x rest newval :: xs)
		  /\
		 (UPDATE_ARRAY_ (x::xs) (SUC i :: rest) newval
			=
			x :: UPDATE_ARRAY_ xs (i::rest) newval)`--)
		;

val STORE2int  = Define `STORE2int (INT i) = i`   ;
val STORE2bool = Define `STORE2bool (BOOL b) = b` ;

val Gt__def = Define
	`Gt_ i j   = BOOL (STORE2int i > STORE2int j)`
	;
val And__def = Define
	`And_ p q = BOOL (STORE2bool p /\ STORE2bool q)`
	;

val Plus__def = Define
	`Plus_ i j   = INT (STORE2int i + STORE2int j)`
	;

val SEM_def = xDefine "SEM_def"
	`(Expr_SEM (Lit (LitInt i))  mem = INT i)		/\
	 (Expr_SEM (Lit (LitBool b)) mem = BOOL b)		/\
	 (Expr_SEM (Store s)   mem = Store_SEM s mem)  		/\
	 (Expr_SEM (Gt x y)    mem = Gt_   (Expr_SEM x mem) (Expr_SEM y mem)) 	/\
	 (Expr_SEM (And x y)   mem = And_  (Expr_SEM x mem) (Expr_SEM y mem)) 	/\
	 (Expr_SEM (Plus x y)  mem = Plus_ (Expr_SEM x mem) (Expr_SEM y mem)) 	
	  /\
	 (ExprList_SEM ExprNil mem = []) 
	  /\
	 (ExprList_SEM (ExprCons e rest) mem
		=
		Expr_SEM e mem :: ExprList_SEM rest mem)
	  /\
	 (Store_SEM (StoreVar ident) mem 
		= 
		Ident_SEM ident mem)			
	  /\
	 (Store_SEM (ArrayApp a j) mem 
		=
		GET_ARRAY_ELEM 
		   (Num (STORE2int (Expr_SEM j mem)))
		   (Store_SEM a mem))	
	  /\
	 (Store_SEM (ArrayUpdate a indices newval) mem
		=
		UPDATE_ARRAY 
		   (Store_SEM a mem)
		   (MAP (Num o STORE2int) (ExprList_SEM indices mem))
		   (Expr_SEM newval mem))
	  /\
	 (Ident_SEM (Identifier name status ty) mem
		=
		mem name)`
	;
	
val get_TopName_def = Define
	`(get_TopName (StoreVar (Identifier name status ty)) = name) 
	  /\
	 (get_TopName (ArrayApp a e) = get_TopName a)
	  /\
	 (get_TopName (ArrayUpdate a indices x) = get_TopName a)` 
	;

val get_Indices_def = Define
	`(get_Indices (StoreVar v)   rest = rest) 
	  /\
	 (get_Indices (ArrayApp a i) rest = get_Indices a (i::rest))`
	;

val Stmt_SEM = Define
	`(Stmt_SEM Skip mem = mem) 
	  /\
	 (Stmt_SEM (Asg (StoreVar x) e) mem 
		=
		(\v. if v = get_TopName (StoreVar x)
		     then Expr_SEM e mem
		     else mem v)
		)
	  /\
	 (Stmt_SEM (Asg (ArrayApp a i) e) mem 
		=
		(\v. if v = get_TopName a
		     then UPDATE_ARRAY 
				(mem (get_TopName a))
				(MAP (Num o STORE2int o (\e. Expr_SEM e mem)) (get_Indices a [i]))
				(Expr_SEM e mem)
		     else mem v)
		)`
	;



val HOA_def = Define `HOA stmt p q = !s. p s ==> q (stmt s)` ;


prove this::

(1)
HOA (Stmt_SEM (Asg (StoreVar v) e)) 
	(Expr_SEM (SUBST q v e))
	(Expr_SEM q)

(2)

HOA (Stmt_SEM (Asg (ArrayApp a i) e)) 
	(Expr_SEM (SUBST q (getframe a) (ArrayUpdate (getframe a) (getIndices a i) e))
	(Expr_SEM q)






	
val _ = export_theory() ; 
