(* //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //======================================================================== *)
(* // *)
(* ============================================================
--
--
-- Authors : Ade Azurat & Wishnu Prasetya
-- Created : feb 2003
============================================================ *)

(* open Parse HolKernel boolLib bossLib  ; *)

load "integerTheory" ;
load "stringTheory" ;
load "listTheory" ;

open integerTheory stringTheory listTheory ;
  
val _ = new_theory "Subst" ;

(* ============================================================
--
-- 
-- 
============================================================ *)

(* underspecified Literal and Type *)

val _ = Hol_datatype
	`Expr =
		  Lit	of 'literal
		| Ident of string => 'type
		| And 	of Expr => Expr
		| Equal of Expr => Expr	
		| Cond	of Expr => Expr => Expr
		| ForAll of string => 'type => Expr => Expr` ;

(*

bvars : list of bound varibles, each with its type

astate : abstract state

*)

val EQUIV_def = Define
	`(EQUIV (Lit d) (Lit e) bvars astate = (d = e)) 
	  /\
	 (EQUIV p (Ident v ty) bvars astate 
		= 
		if MEM (v,ty) bvars
		then (p = Ident v ty)
		else (p = astate v ty))
	  /\
	 (EQUIV (And d1 d2) (And e1 e2) bvars astate 
		=
		EQUIV d1 e1 bvars astate  /\  EQUIV d2 e2 bvars astate)
	  /\
	 (EQUIV (Equal d1 d2) (Equal e1 e2) bvars astate 
		=
		EQUIV d1 e1 bvars astate  /\  EQUIV d2 e2 bvars astate)
	  /\
	 (EQUIV (Cond g1 d1 d2) (Cond g2 e1 e2) bvars astate 
		=
		EQUIV g1 g2 bvars astate  /\  
		EQUIV d1 e1 bvars astate  /\  
		EQUIV d2 e2 bvars astate)
	  /\
	 (EQUIV (ForAll u tyu d1 d2) (ForAll  v tyv e1 e2) bvars astate 
		=
		(u=v) /\ (tyu = tyv) /\
		EQUIV d1 e1 ((u,tyu)::bvars) astate  /\  
		EQUIV d2 e2 ((u,tyu)::bvars) astate)
	`
	;


val FreshName_def = Define
	`(FreshName x (Lit d) = T)
	  /\
	 (FreshName x (Ident v ty) = ~(x=v))
	  /\
	 (FreshName x (And d1 d2) = FreshName x d1 /\ FreshName x d2)
	  /\	
	 (FreshName x (Equal d1 d2) = FreshName x d1 /\ FreshName x d2)
	  /\
	 (FreshName x (Cond g d1 d2) 
		= 
		FreshName x g /\ FreshName x d1 /\ FreshName x d2)
	  /\
	 (FreshName x (ForAll u ty d1 d2)
		=
		~(x=u)	/\ FreshName x d1 /\ FreshName x d2)
	`
	;

val _ = new_constant ("primeit" , Type `:string->string`) ;

val PrimeIt_def = Define
	`(PrimeIt x tyx (Lit d) bvars = Lit d)
	  /\
	 (PrimeIt x tyx (Ident v tyv) bvars
		=
		if (x=v) /\ (tyx = tyv) /\ ~(MEM (x,tyx) bvars) 
		then Ident (primeit x) tyx
		else Ident v tyv)
	  /\
	 (PrimeIt x tyx (And d1 d2) bvars 
		= 
		And (PrimeIt x tyx d1 bvars) (PrimeIt x tyx d2 bvars))
	  /\
	 (PrimeIt x tyx (Equal d1 d2) bvars 
		= 
		Equal (PrimeIt x tyx d1 bvars) (PrimeIt x tyx d2 bvars))
	  /\
	 (PrimeIt x tyx (Cond g d1 d2) bvars
		= 
		Cond (PrimeIt x tyx g bvars) 
			(PrimeIt x tyx d1 bvars) 
			(PrimeIt x tyx d2 bvars))
	/\
	(PrimeIt x tyx (ForAll u ty d1 d2) bvars
		=
		ForAll u ty
			(PrimeIt x tyx d1 ((u,ty)::bvars))
			(PrimeIt x tyx d2 ((u,ty)::bvars)))
	`
	;


val Subst_def = Define
	`(Subst (Lit c) target rhs bvars = Lit c)
	  /\
	 (Subst (Ident  v tyv) (Ident x tyx) rhs bvars
		=
		if (x=v) /\ (tyx = tyv) /\ ~(MEM (v,tyv) bvars) 
		then rhs 
		else Ident v tyv)
	  /\
 	 (Subst (And d1 d2) target rhs bvars
		=
		And (Subst d1 target rhs bvars) (Subst d2 target rhs bvars))
	  /\
 	 (Subst (Equal d1 d2) target rhs bvars
		=
		Equal (Subst d1 target rhs bvars) (Subst d2 target rhs bvars))
	  /\
 	 (Subst (Cond g d1 d2) target rhs bvars 
		=
		Cond (Subst g target rhs bvars) 
			(Subst d1 target rhs bvars) 
			(Subst d2 target rhs bvars))
	  /\
	 (Subst (ForAll u ty d1 d2) target rhs bvars
		=
		ForAll u ty (Subst d1 target rhs ((u,ty)::bvars))
		            (Subst d2 target rhs ((u,ty)::bvars)))
	`
	;


val Subst_th1 = store_thm
	("Subst_thm1",
	 --`!(e:('lit,'type) Expr) bvars.
	  (!v tyv. if (v = x) /\ (tyv = tyx)
			then (astate v tyv = rhs) 
			else (astate v tyv = Ident v tyv)) /\
	  ~MEM (primeit x, tyx) bvars   /\
	  FreshName (primeit x) e
	  ==>
	  EQUIV (Subst e (Ident x tyx) rhs bvars) 
		(PrimeIt x tyx e bvars) 
		bvars 
		astate`--,

	let val defs = 
		[EQUIV_def,
		 Subst_def,
		 PrimeIt_def,
		 FreshName_def]
	in
	Induct THEN RW_TAC std_ss defs
	THEN TRY (MesonLib.GEN_MESON_TAC 20 20 20 [])
	THENL
	[ Induct_on `rhs:Expr`
	  THEN RW_TAC std_ss defs
	  THEN PROVE_TAC [],
	  PROVE_TAC [],
	  FIRST_ASSUM MATCH_MP_TAC THEN RW_TAC list_ss [],
  	  FIRST_ASSUM MATCH_MP_TAC THEN RW_TAC list_ss []	  	  
	]
	end 
	) ;


	
val _ = export_theory() ; 
