(* //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //======================================================================== *)
(* // *)

(* ============================================================ 

** Theorems about arrays, used to hide HOL internal representation of
** array.

============================================================ *)

open Parse HolKernel boolLib bossLib integerTheory  ;
open x_StdConstTheory intLib ;
  
val _ = new_theory "x_Array" ;

val ArrayUpdate_thm1 =
	let
	val thm = prove 
		  (--`!a. (ArrayUpdate a i x) ArrayApp i = x`--,
		     Induct
		     THEN RW_TAC std_ss [ArrayUpdate_def,ArrayApp_def])
	in
	save_thm ("ArrayUpdate_thm1", SPEC_ALL thm) 
	end ;

val ArrayUpdate_thm2 =
	let
	val thm = prove 
		  (--`!a. (ArrayUpdate a i x) ArrayApp j 
			  =
			  if (i=j) then x else a ArrayApp j`--,
		     Induct
		     THEN RW_TAC std_ss [ArrayUpdate_def,ArrayApp_def])
	in
	save_thm ("ArrayUpdate_thm2", SPEC_ALL thm) 
	end ;

val SizeOf_ArrayUpdate_thm =
		let
	val thm = prove 
		  (--`!a. arraySize (ArrayUpdate a i x) = arraySize a`--,
		     Induct
		     THEN RW_TAC std_ss [ArrayUpdate_def,arraySize_def])
	in
	save_thm ("SizeOf_ArrayUpdate_thm", SPEC_ALL thm) 
	end ;


val ArrayDom_thm1 = 
	let
	val thm = prove 
		(--`ArrayDom a i i = []`--,
		 ONCE_REWRITE_TAC [ArrayDom_defora]
		 THEN RW_TAC std_ss [] THEN COOPER_TAC)	
	in
	save_thm ("ArrayDom_thm1",thm)
	end ;

val ArrayDom_thm2 = 
	let
	val thm = prove 
		(--`i>=j ==> (ArrayDom a i j = [])`--,
		 ONCE_REWRITE_TAC [ArrayDom_defora]
		 THEN RW_TAC std_ss [] THEN COOPER_TAC)		 
	in
	save_thm ("ArrayDom_thm2", UNDISCH thm)
	end ;

val ArrayDom_thm3 = 
	let
	val thm = prove	
		(--`ArrayDom a i (i+1) = [a ArrayApp i]`--,
		 ONCE_REWRITE_TAC [ArrayDom_defora]
		 THEN RW_TAC int_ss [ArrayDom_thm1] THEN COOPER_TAC)	
	in
	save_thm ("ArrayDom_thm3",thm)
	end ;

val ArrayDom_ora4 = 
	let
	val ora = mk_oracle_thm 
			(Tag.read "ArrayDom_ora4")
			([--`i<=j`--],
			--`ArrayDom a i (j+1) 
			   = 
			   APPEND (ArrayDom a i j) [a ArrayApp j]`--)

	in
	save_thm ("ArrayDom_ora4",ora)
	end ;

val _ = export_theory() ;
