(* //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //======================================================================== *)
(* // *)

(* ============================================================ 
--
** Contains the HOL definitions of xmech pre-defined constants which
** have not been defined in HOL standard libraries.
--
============================================================ *)

open Parse HolKernel boolLib bossLib integerTheory 
  
val _ = new_theory "x_StdConst" ;


val notEqual_def = Define `notEqual a b = ~(a = b)` ;
val max_def     = Define `max = int_max` ;
val min_def     = Define `min = int_min` ;
val MEM__def    = Define `MEM_ = MEM` ;
val APPEND__def = Define `APPEND_ = APPEND` ;

val _ = add_infix ("notEqual",100,Parse.NONASSOC) ;
val _ = add_infix ("max",300,Parse.RIGHT) ;
val _ = add_infix ("min",300,Parse.RIGHT) ;
val _ = add_infix ("MEM_",100,Parse.NONASSOC) ;
val _ = add_infix ("APPEND_",110,Parse.RIGHT) ;

val SUM__def = Define 
	`(SUM_ []    = 0) /\
	 (SUM_ (x::s) = x + SUM_ s)` ;

val COUNT_def = Define
	`(COUNT [] = 0) /\
	 (COUNT (x::s) = 1 + COUNT s)` ;

val MAX_def = Define
	`(MAX [x] = x) /\
	 (MAX (x::s) = x max MAX s)` ;

val MIN_def = Define
	`(MIN [x] = x) /\
	 (MIN (x::s) = x min MIN s)` ;

val MAXP_def = Define
	`(MAXP [] = 0) /\
	 (MAXP (x::s) = x max MAXP s)` ;

(* ============================================================
--
-- Array
-- 
============================================================ *)

val _ = Hol_datatype `ARRAY = ARRAY of int => (int->'a)` ;

val arraySize_def = Define `arraySize (ARRAY n c) = n` ;

val ArrayApp_def = Define `ArrayApp (ARRAY n c) i = c i` ;

val _ = add_infix ("ArrayApp",310,Parse.LEFT) ;

val ArrayUpdate_def 
	=
	Define `ArrayUpdate (ARRAY n c) i x
		=
		ARRAY n (\j. if j=i then x else c j)` ;


val FromTo_def   = Define `FromTo i left right = (left<=i) /\ (i<right)` ;

val _ = new_constant 
	("ArrayDom",
	 Type `:'a ARRAY -> int -> int -> 'a list`) ;

val ArrayDom_defora 
	=
	let
	val ora = mk_oracle_thm 
			(Tag.read "ArrayDom_defora")
			([],--`ArrayDom a lb ub 
				=
	 			if ub<=lb 
				then []
				else (a ArrayApp lb) :: ArrayDom a (lb+1) ub`--) 
	in
	save_thm ("ArrayDom_defora",ora)
	end ;

val MARKER_def = Define `MARKER info x = x` ;

val lprod__def 
	=
	Define
	`(lprod_ x [] = [])	
	  /\
	 (lprod_ x (y::t) = (x,y) :: lprod_ x t)` ;

val LPROD_def 
	= 
	Define
	`(LPROD [] t  = [])
	  /\
	 (LPROD (x::s) t = APPEND (lprod_ x t) (LPROD s t))` ;

val _ = add_infix ("LPROD",110,Parse.RIGHT) ;
 
val SQLSELECT_def
	=
	Define
	`SQLSELECT f p joined_databases
		=
		MAP f (FILTER p joined_databases)` ; 

	
val _ = export_theory() ; 
