-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //
module Configuration.Lang(
	  default_EXPR_prefixes 
	, default_EXPR_unaries 
	, default_EXPR_infixes 
	, default_EXPR_quantors 
	, default_simple_EXPR_prefixes 
	, default_simple_EXPR_unaries
	, default_simple_EXPR_infixes
	, default_simple_EXPR_quantors
	, default_EXPR_constants 
	, default_simple_EXPR_constants 
	, is_specSym
	, is_keywOp
	, additional_EXPR_keywords 
	, additional_EXPR_keywordops 
	)
where

import PrimitiveConcepts.LC
import Configuration.DefaultEXPRconsts
import MechScanner.PCKG
import Lang.TYPE.Parsers
import SupportLib.MoreParser
import Maybe

scanner = stringScanner scannerspec
parser info tyspec
	=
	if null errors 
	then result
	else error ("Parsing failure in the type specification of "
		    ++ info
		    ++ ".\n Check it in the list of the predefined constants.")

	where
	--
	-- calling string parser ...uhm... using fromJust to cast 
	-- back from monad
	--
	(result,errors) = fromJust (parseString scanner pType tyspec)

simple_only = filter p
	where
	p c = lc_name c `elem` simple_

s `subset` t = all (`elem` t) s

default_EXPR_prefixes = map f prefixes
	where
	f (n,ty,n_) = UserDefinedLogicalConstant 
			n
			(parser n ty)
			Prefix
			n_

default_EXPR_unaries = map f unaries
	where
	f (n,ty,n_) = UserDefinedLogicalConstant 
			n
			(parser n ty)
			Prefix
			n_

default_EXPR_infixes = map f infixes
	where
	f (n,ty,fix,n_) = UserDefinedLogicalConstant n (parser n ty) fix n_

default_EXPR_quantors = map f quantors
	where
	f (n,ty,n_) = UserDefinedLogicalConstant 
			n
			(parser n ty)
			Quanty
			n_

default_simple_EXPR_prefixes = simple_only default_EXPR_prefixes
default_simple_EXPR_unaries = simple_only default_EXPR_unaries
default_simple_EXPR_infixes = simple_only default_EXPR_infixes
default_simple_EXPR_quantors = simple_only default_EXPR_quantors

default_EXPR_constants 
	= 
	   default_EXPR_prefixes
	++ default_EXPR_unaries
	++ default_EXPR_infixes
	++ default_EXPR_quantors

default_simple_EXPR_constants 
	= 
	   default_simple_EXPR_prefixes
	++ default_simple_EXPR_unaries
	++ default_simple_EXPR_infixes
	++ default_simple_EXPR_quantors

is_specSym [c] = c `elem` specialchars
is_specSym _ = False

is_keywOp name = name `subset` opchars

--
--Calculate additional keywords that would be required
--
additional_EXPR_keywords 
	=
	filter p . map lc_name 
	$
	(  default_EXPR_unaries
	++ default_EXPR_infixes
	++ default_EXPR_quantors)

	where

	p name = not (is_specSym name || is_keywOp name)

additional_EXPR_keywordops 
	=
	filter is_keywOp . map lc_name 
	$
	(  default_EXPR_unaries
	++ default_EXPR_infixes
	++ default_EXPR_quantors)


