-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //
module Engine.SyntaxChecker.ErrorMsg 

where

--import UU.Scanner(Pos(..))
import PrimitiveConcepts.Position
import PrimitiveConcepts.ErrorMsg
import UU.Pretty
import Configuration.Face
import Etcetera.PCKG

data SyntaxError =
     SyntaxError 
	(Maybe Pos)		--position of the error
	PP_Doc			--the term in error
	(Maybe PP_Doc)		--enclosing term
	PP_Doc			--explanation

type SyntaxErrorList = [SyntaxError]

ppSyntaxError (SyntaxError pos ppexpr ppinexpr moreinfo)
	=
	errorMsgBox pos header (msg_encloser >-< msg)
	where
	msg_encloser	= case ppinexpr of
			  Just ppe	-> text "somewhere in construct:"
					   >-< indent_ ppe
					   >-< "More specifically look at:"
			  _		-> empty

	msg 	= indent_ ppexpr >-< moreinfo

	header  = "Syntax error"
	indent_ = indent std_indentWidth

ppSyntaxErrorList []  = text "No syntax error found."
ppSyntaxErrorList [e] = ppSyntaxError e 
ppSyntaxErrorList es  = pp_ (zip [1..] es)
	where

	pp_ ((n,e):es) = ppe n e 
			 >-<
			 foldl (>-<) empty (map ppe_ es)

	ppe n e = ppi n >#< ppSyntaxError e

	lwide = (length . show . length $ es) + 2
	ppi n = text . take lwide $ ("[" ++ show n ++ "]" ++ repeat ' ')

	ppe_ (n,e) = text "" >-< ppe n e

sortSyntaxErrorList s = gensort less s
	where
	(SyntaxError Nothing _ _ _) `less` _  = True
	(SyntaxError (Just (Pos l1 _ _)) _ _ _) 
		`less` 
		(SyntaxError (Just (Pos l2 _ _)) _ _ _) = l1 < l2
	_ `less` _ = False

syntaxError_illegalName1 decl name 
	=
	SyntaxError
		(Just (getPos name))
		(pp name)
		(Just (pp decl))
		info
	where
	info = text "The declared name contains __ (which is not allowed)."

syntaxError_illegalName2 name 
	=
	SyntaxError
		(Just (getPos name))
		(pp name)
		Nothing
		info
	where
	info = text "The declared name contains __ (which is not allowed)."

syntaxError_multiplyDeclaredIdent1 decl name 
	=
	SyntaxError
		(Just (getPos name))
		(pp name)
		(Just (pp decl))
		syntaxError_multiplyDeclaredIdent_info 

syntaxError_multiplyDeclaredIdent2 name 
	=
	SyntaxError
		(Just (getPos name))
		(pp name)
		Nothing
		syntaxError_multiplyDeclaredIdent_info

syntaxError_multiplyDeclaredIdent_info 
	=
	info1 >-< info2 >-< info3 >-< info4
	where
	info1 = text "which is multiply declared in the same scope."
	info2 = text "Perhaps the name has been declared before."
	info3 = text "Perhaps the name is a pre-defined logical constant."
	info4 = text "If it's the counter of a for-loop, perhaps it's also used as the loop termination counter."


syntaxError_off_namespace ident
	=
	SyntaxError
		(Just (getPos ident))
		(pp ident)
		Nothing
		(info1 >-< info2 >-< info3 >-< info4)
	where
	info1 = text "The name is either unknown or not allowed in the namespace at that location."
	info2 = text "Perhaps it should be a program variable or a logical constant, but its declaration is missing."
	info3 = text "Perhaps it should be a bound variable, but its binding is missing."
	info4 = text "Otherwise, consult the definition of allowed namespace."


syntaxError_writingROVar identname asg
	=
	SyntaxError
		(Just (getPos asg))
		(pp (text identname))
		(Just (pp asg))
		(info1 >-< info2)
	where
	info1 = text "which is a readonly variable."
	info2 = text "Assignment to a readonly variable is not allowed."

syntaxError_writingForLoopCounter identname asg
	=
	SyntaxError
		(Just (getPos asg))
		(pp (text identname))
		(Just (pp asg))
		(info1 >-< info2)
	where
	info1 = text "which is a for-loop counter variable."
	info2 = text "Assignment to a such a variable is not allowed."