-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //
module Engine.TypeChecker.Basic

where

import UU.Pretty(PP_Doc(..))
import PrimitiveConcepts.Position
import Lang.TYPE.PCKG
import Maybe
import List(nub)

--
-- a type scheme here is either just a flat type or a type where
-- all type variables are universally bound to a ForAllTy binder.
--
data TypeScheme =
	  FlatTy	Type
	| ForAllTy 	Type

--
-- getting free type variables of a type specification
--
get_ftv ty = nub (ftv ty)
	where
	ftv ty = case ty of
		TyVar v		-> [v]
		TyCons _ tys	-> concat . map ftv $ tys
		TyRecord fields -> concat . map (ftv . get_Field_ty) $ fields
		TyUnknown	-> []

	get_Field_ty (Field _ ty) = ty

--
-- Converting to type-scheme
--
toTypeScheme ty 
	= 
	if null (get_ftv ty)
	then FlatTy ty
	else ForAllTy ty

--
-- substituting a type variable
--
substTy (tvar,ty) ty' = subst ty'
	where
	subst ty' = case ty' of
		TyVar v		-> if v==tvar
				   then ty
				   else ty'
		TyCons c tys	-> TyCons c (map subst tys)
		TyRecord fields	-> TyRecord (map subst_f fields)
		TyUnknown	-> ty'

	subst_f (Field fname ty) = Field fname (subst ty)

--
-- substituting with a list of substitutions
--
substituteTy subs ty = foldr substTy ty subs


--
-- Creating a fresh type variable.
-- Convention: a__(number)
--
mk_freshTyVar i = TyVar ("__a" ++ show i)

--
-- instantiating a type scheme with fresh type variables
--
freshInstantiateTy freshname (FlatTy ty)     = (freshname,ty)
freshInstantiateTy freshname (ForAllTy ty) = (next_freshname,ty')
	where
	free_tvars     = get_ftv ty
	new_tvars      = map mk_freshTyVar [freshname..]
	substitution   = zip free_tvars new_tvars 
	next_freshname = freshname + length free_tvars

	ty' = substituteTy substitution ty


--
-- Identifier context
--
type IdentContext = [(String,TypeScheme)]

--
-- updating context
--
updateContext updates context = updates ++ context_
	where
	context_ = filter p context
	p (v,_) = isNothing (lookup v updates)

--
-- Type Constraint error info
--
data TypeConstraintError 
	= 
	  TCmismatch 
		(Maybe Pos)
		PP_Doc		--this expression
		(Maybe PP_Doc)	--used in this construct
		Type		--which has this type 
		Type		--does not match this inferred type
		PP_Doc		--additional information about the
				--reason of type error

type TypeConstraintErrorList = [TypeConstraintError]

--
-- Constraint and constraint list
--

type Constraint__List = [Constraint_]

data Constraint_ =  TyEquiv_ TypeConstraintError Type Type

t1 <==> t2 = (\info-> TyEquiv_ info t1 t2)


--
-- Type variable context
--
type TyVarContext = [(String,Type)]
