-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //
module Engine.TypeChecker.Bridge2CSolver where

import Lang.TYPE.PCKG
import Engine.TypeChecker.Basic
import Maybe
import List(nub)

--
-- Importing Bastiaan's constraint solver
--
import Constraints
import ConstraintInfo
import Types
import SolveGreedy

--
--converting our type representation to Bastiaan's representation
--
toSolverType nameTable ty = translate ty
	where
	translate ty = case ty of
	  TyVar v		-> ( case lookup v nameTable of
					Just i -> TVar i
					_          -> error "booo!" )

	  TyCons tc ts    	-> TApp (TApp (TCon "__TyCons") (TCon tc))
					(translist ts)

	  TyRecord fields	-> TApp (TCon "__Record") (transfields fields)

	  TyUnknown		-> TCon "__TyUnknown"

	transfields fields = foldr op e fields
		where
		e = TCon "__NomoreField"
		op (Field fn t) r = TApp (TApp (TCon "__MoreField") 
					      (TApp (TCon fn) (translate t)))
					  r

	translist ts = foldr op e ts
		where
		e = TCon "__Nil"
		op t r = TApp (TApp (TCon "__Cons") (translate t))
					  r
--
--this is then the reverse conversion (hope the solver maintain 
--the bijection)
--
fromSolverType nameTable ty = translate ty
	where
	--
	-- assume names in nameTable is indexed from 0.. 
	--
	translate ty = case ty of
	   TVar i		-> TyVar . fst $ (nameTable!!i)

	   TCon "__TyUnknown"	-> TyUnknown

	   TApp (TCon "__Record") t -> TyRecord (trans2fields t)

	   TApp (TApp (TCon "__TyCons") (TCon tc)) t
			-> 
			TyCons tc (trans2list t)

	trans2fields (TCon "__NomoreField") = []
	trans2fields (TApp (TApp (TCon "__MoreField") (TApp (TCon fn) tf)) t)
		= 
		Field fn (translate tf) : trans2fields t

	trans2list (TCon "__Nil") = []
	trans2list (TApp (TApp (TCon "__Cons") t1) trest)
		= 
		(translate t1) : trans2list trest

--
--For building a name table. This is because in Bastiaan's representation
--tyvar is represented by integer, and in ours with string.
--
mk_nameTable constraints = zip names [0..]
	where
	names = collectTvars constraints

--
--Just a help function to collect all tvars from a list of constraints
--
collectTvars constraints = nub . concat . map collect $ constraints
	where
	--
	-- due to some optimization, info may contains more type vars
	-- than t1+t2 ... and vice versa, so we have to search in both
	--

	collect (TyEquiv_ info t1 t2) = col_ info ++ col t1 ++ col t2

	col_ (TCmismatch _ _ _ ty tyinfer _) = col ty ++ col tyinfer

	col (TyVar v) = [v]
	col (TyCons tc ts) = concat (map col ts)
	col (TyRecord fields) = concat (map colf fields)

	colf (Field _ t) = col t

--
-- Translate our representation of typing constraints to Bastiaan's
--
translateConstraints nameTable constraints = map t constraints
	where
	t (TyEquiv_ errMsg t1 t2) 
		= 
		Equiv errMsg 
			(toSolverType nameTable t1) 
			(toSolverType nameTable t2)

--
-- dummy instantiation to this class
-- (method class does nothing)
-- (our substitution does not map very well to Bastiaan's, in particular
--  the type of ftv .. cause we'll need to carry around the name table
--  for translation)
--
instance Substitutable TypeConstraintError
--
-- also dumy instantiation here
-- still not sure what the methods of this class are supposed to do
--
instance ConstraintInfo TypeConstraintError 

--
-- An interface for calling the constraint solver
--
solveconstraints unique constraints = (errors,subs)

	where

	(_,subs,errors,_) = solveGreedy unique options constraints
	
	options = []

--
-- Applying substitution from solver to our type
--
applySubs nameTable subs_from_solver ty
	=
	fromSolverType nameTable 
	. (subs_from_solver |-> ) 
	. toSolverType   nameTable
	$ ty

--
-- Applying substitutions from solver back to constraint
-- (in particular for better error messages)
--
applySubsToConstraint nameTable 
	subs_from_solver 
	(TCmismatch pos ppexpr ppinexpr ty tyinfer moreinfo)

	=

	TCmismatch pos ppexpr ppinexpr ty' tyinfer' moreinfo

	where

	ty'       = applySubs nameTable subs_from_solver ty
	tyinfer'  = applySubs nameTable subs_from_solver tyinfer
