-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //
module Engine.TypeChecker.ErrorMsg 

where

import UU.Pretty
import Configuration.Face
import PrimitiveConcepts.Position
import PrimitiveConcepts.ErrorMsg
import Lang.TYPE.PCKG
import Engine.TypeChecker.Basic


-- indent_ :: PP PP_Doc => PP_Doc -> PP_Doc
indent_ :: PP_Doc -> PP_Doc
indent_ = indent std_indentWidth

header        = "Conflicting type constraint"
enbox pos msg = errorMsgBox pos header msg


ppTypeConstraintError (TCmismatch pos ppexpr ppinexpr ty tyinfer moreinfo)
	=
	enbox pos msg
	where
	msg_ = case ppinexpr of
		 Just ppe	-> text "somewhere in construct:"
				   >-<
				   indent_ ppe
				   >-<
				   text "More specifically, look "
		 _		-> empty

	msg = msg_
		>|< "in construct:" 
		>-< indent_ ppexpr  
		>-< text "which (here) seems to have the type:" 
		>-< indent_ (pp ty)   
		>-< text "but it is used as if it has the type:" 
		>-< indent_ (pp tyinfer)
		>-< moreinfo


ppTypeConstraintErrorList []  = text "No type error found."
ppTypeConstraintErrorList [e] = ppTypeConstraintError e 
ppTypeConstraintErrorList es  = pp_ (zip [1..] es)
	where

	pp_ ((n,e):es) = ppe n e 
			 >-<
			 foldl (>-<) empty (map ppe_ es)

	ppe n e = ppi n >#< ppTypeConstraintError e

	lwide = (length . show . length $ es) + 2
	ppi n = text . take lwide $ ("[" ++ show n ++ "]" ++ repeat ' ')

	ppe_ (n,e) = text "" >-< ppe n e
	
--
-- A general way to construct type error msg:
-- type of expression e does not match its inferred type
--
typeError exp inexp ty infer_ty moreinfo
	=
	TCmismatch
		(Just (getPos exp))
		(pp exp) 
		(Just (pp inexp))
		ty 
		infer_ty
		moreinfo

--
-- special type error for function aplication like construct
--
typeError_App expf inexp ty infer_ty 
	=
	TCmismatch
		(Just (getPos expf))
		(pp expf) 
		(Just (pp inexp))
		ty 
		infer_ty
		(text "Perhaps the construct is not a function."
		 >-<
		 text "Perhaps the construct's type conflicts with its argument's type.")
 
typeError_ArrayApp expa inexp ty infer_ty 
 	=
	TCmismatch
		(Just (getPos expa))
		(pp expa) 
		(Just (pp inexp))
		ty 
		infer_ty
		(text "Perhaps the construct is not an array.")

typeError_ListElement exp inlist ty infer_ty 
 	=
	TCmismatch
		(Just (getPos exp))
		(pp exp) 
		(Just (pp inlist))
		ty 
		infer_ty
		cause
	where
	cause  = text "Either this list element has a wrong type, or another element does."
		 >-<
		 text "(a list's elements should have a uniform type)"
		 
typeError_FromTo ranger_exp inexp ty infer_ty 
 	=
	TCmismatch
		(Just (getPos ranger_exp))
		(pp ranger_exp) 
		(Just (pp inexp))
		ty 
		infer_ty
		cause
	where
	cause  = text "Perhaps the construct is not an Int (nor an Array)."


typeError_Record rexp inexp ty infer_ty 
 	=
	TCmismatch
		(Just (getPos rexp))
		(pp rexp) 
		(Just (pp inexp))
		ty 
		infer_ty
		(cause1 >-< cause2 >-< cause3)
	where
	cause1  = text "Perhaps the conflict is in the type of the field."
	cause2  = text "Perhaps the field does not occur in the record."
	cause3  = text "Perhaps the construct is not a record."

typeError_Record_2 rexp ty infer_ty 
 	=
	TCmismatch
		(Just (getPos rexp))
		(pp rexp) 
		Nothing
		ty 
		infer_ty
		(cause1 >-< cause2 >-< cause3)
	where
	cause1  = text "Perhaps the conflict is in the type of the field."
	cause2  = text "Perhaps the field does not occur in the record."
	cause3  = text "Perhaps the construct is not a record."


typeError_Generator gen ty infer_ty 
 	=
	TCmismatch
		(Just (getPos gen))
		(pp gen) 
		Nothing
		ty 
		infer_ty
		(cause1 >-< cause2)
	where
	cause1  = text "Perhaps the generator is not a list."
	cause2  = text "Perhaps the generator is a list of a wrong type."


typeError_notBool exp ty
	= 
	TCmismatch
		(Just (getPos exp))
		(pp exp)
		Nothing
		ty
		boolTy
		empty

typeError_notInt exp ty
	= 
	TCmismatch
		(Just (getPos exp))
		(pp exp)
		Nothing
		ty
		intTy
		empty

--
-- special type error in application of binary operator
--
typeError_Binary direction op expr tyop infer_tyop 
	= 
	TCmismatch
		(Just (getPos op))
		(pp op)
		(Just (pp expr))
		tyop
		infer_tyop
		(cause1 >-< cause2)

	where

	cause1 = text "Perhaps the operator's"
		 >#<
		 text direction
		 >#< text "argument has a wrong type."

	cause2 = text "Perhaps it is a wrong operator to use here."
	

typeError_BinaryL :: (ContainPos i, PP i, PP e) => i->e->Type->Type->TypeConstraintError
typeError_BinaryL = typeError_Binary "left"
typeError_BinaryR :: (ContainPos i, PP i, PP e) => i->e->Type->Type->TypeConstraintError
typeError_BinaryR = typeError_Binary "right"

typeError_thenelse texpr iteexpr ty infer_ty
	=
	TCmismatch
		(Just (getPos iteexpr))
		(pp texpr)
		(Just (pp iteexpr))
		ty
		infer_ty
		(text "Perhaps the types of the then- and else- constructs are not the same.")


typeError_SingleAssignment rhs_asg asg ty_rhs ty_lhs
	= 
	TCmismatch
		(Just (getPos asg))
		(pp rhs_asg)
		(Just (pp asg))
		ty_rhs
		ty_lhs
		(text "The type of the rhs of the assignment conflicts with its lhs.")

--test :: PP PP_Doc => PP_Doc -> IO()
--test pp = putStr (disp pp 90 "")

