-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //
module Engine.VCG.Basic(
	 module Engine.VCG.Basic
	,module Engine.VCG.Marker
	)

where

import Lang.TYPE.PCKG
import Lang.EXPR.PCKG
import Engine.VCG.Marker
import Engine.Substitution.PCKG

--
-- Reduction mode, and functions to plug in and out sub-vcgs
--
data ReduceOption = 
	  RO_AOC	-- to get vcs for array overflow check 
	| RO_DZC	-- to get vcs for division by zero check
	| RO_T		-- to get vcs for termination check
	| RO_ATotal	-- to get vcs for abstracted, total correctness
	| RO_APartial	-- to get vcs for abstracted, partial correctness
	| RO_All	-- to get all vcs
	deriving (Eq,Show)

plugIn_StandardVCG mode conditions 
	=
	if mode `elem` [RO_All
			,RO_ATotal
			,RO_APartial
			,RO_T]
	then conditions
	else []

plugIn_AOCVCG mode conditions
	=
	if mode `elem` [RO_All,RO_AOC]
	then conditions
	else []

plugIn_DZCVCG mode conditions
	=
	if mode `elem` [RO_All,RO_DZC]
	then conditions
	else []
--
-- Note that RO_T and RO_Partial are not implemented as plug-ins
-- (sorry, i'm a bit too lazy to do that)
--

--
-- Some reduction-mode-dependent post-processing and/of post filtering of the vcs.
-- 
postprocess_vcs option vcs = case option of

	 RO_AOC 	-> map unmark vcs
	 RO_DZC		-> map unmark vcs
	 RO_APartial	-> filter (notMarkedBy [TC1,TC2]) vcs
	 RO_T		-> filter (not . notMarkedBy [TC1,TC2]) vcs 
	 otherwise	-> vcs


--
-- This is the function implementing the wp of assignment
--
wp_asg (target,e) q = subst [(a,e')] q
	where
	(a,e') = case target of

			Ident (Identifier name _ _ _)	-> (name,e)
			ArrayApp  _ a i			-> f e [i] a
			RecordApp _ r n			-> g e [n] r

	f e s (ArrayApp _ a i) = f e (i:s) a
	f e s i@(Ident (Identifier name _ _ _)) = (name,ArrayUpdate i (reverse s) e)
	f e s q@(RecordApp _ r n) = g (ArrayUpdate q (reverse s) e) [n] r

	g e s (RecordApp _ r n) = g e (n:s) r
	g e s i@(Ident (Identifier name _ _ _)) = (name,RecordUpdate i (reverse s) e TyUnknown)
	g e s a@(ArrayApp _ b i) = f (RecordUpdate a (reverse s) e TyUnknown) [i] a



imp__ (Lit (LitBool True _)) q = q
imp__  p q                     = imp_ p q

