-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //
module Lang.EXPR.Util

where

import EXPR_data
import UU.Pretty
import PrimitiveConcepts.Position
import qualified EXPR_tree
import EXPR_pp 
import Configuration.Face
import Lang.TYPE.PCKG

instance Show Ident
  where
  showsPrec _ e s = disp (EXPR_tree.ppIdent False e)  std_PageWidth s

instance Show Expr
  where
  showsPrec _ e s = disp (EXPR_tree.ppExpr False e)  std_PageWidth s

instance Show ListGenerator
  where
  showsPrec _ g s = disp (EXPR_tree.ppListGenerator False g)  std_PageWidth s  

ppIdentTree = EXPR_tree.ppIdent

instance PP Ident 
  where
  pp = ppIdent 

ppExprTree = EXPR_tree.ppExpr

instance PP Expr 
  where
  pp = ppExpr 

ppListGeneratorTree = EXPR_tree.ppListGenerator

instance PP ListGenerator
  where
  pp = ppListGenerator 

instance ContainPos Ident where
  getPos (Identifier _ _ _ pos) = pos

instance ContainPos Literal where
  getPos lit = case lit of
	(LitInt _ pos)		-> pos
	(LitBool _ pos)		-> pos
	(LitString _ pos)	-> pos

instance ContainPos Expr where
  getPos expr = case expr of
	(Lit lit) 		-> getPos lit
	(Ident i) 		-> getPos i
	(Cond pos _ _ _)	-> pos
	(App _ e')		-> getPos e'
	(Unary op _)		-> getPos op
	(Binary op _ _)		-> getPos op
	(Quantor op _ _ _)	-> getPos op
	(ArrayApp pos _ _)	-> pos
	(List pos _)		-> pos
	(FromTo	pos _ _ _)	-> pos
	(Record	pos _)		-> pos
	(RecordApp pos _ _)	-> pos
	(SQLSelect pos _ _ _)	-> pos
	(Marker _ e')		-> getPos e'
	_			-> dummyPos

instance ContainPos ListGenerator where
  getPos (ListGenerator p v g) = p

zero = Lit (LitInt 0 dummyPos )
one  = Lit (LitInt 1 dummyPos )
tt   = Lit (LitBool True dummyPos)
ff   = Lit (LitBool False dummyPos)

and_ e1 e2 = Binary op e1 e2
	where
	op = Identifier "/\\" LogicIdent ty_b_b_b dummyPos

imp_ p q = Binary op p q
	where
	op = Identifier "==>" LogicIdent ty_b_b_b dummyPos

or_ p q = Binary op p q
	where
	op = Identifier "\\/" LogicIdent ty_b_b_b dummyPos

not_ p = Unary op p
	where
	op = Identifier "~" LogicIdent ty_b_b dummyPos

forall_ vars dom body = Quantor op vars dom body
	where
	op = Identifier "!" LogicIdent ty_b_b dummyPos

lt_ x y = Binary op x y
	where
	op = Identifier "<" LogicIdent ty_i_i_b dummyPos

lte_ x y = Binary op x y
	where
	op = Identifier "<=" LogicIdent ty_i_i_b dummyPos

gt_ x y = Binary op x y
	where
	op = Identifier ">" LogicIdent ty_i_i_b dummyPos

plus_ x y = Binary op x y
	where
	op = Identifier "+" LogicIdent ty_i_i_i dummyPos

minus_ x y = Binary op x y
	where
	op = Identifier "-" LogicIdent ty_i_i_i dummyPos

eqint_ x y = Binary op x y
	where
	op = Identifier "=" LogicIdent ty_i_i_b dummyPos






