-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //

module Lang.SEQ.Common_Parser(p_SeqProg)

where

import UU.Scanner hiding (pKey,pSpec)
import qualified UU.Scanner (pKey,pSpec)
import UU.Parsing 
import Lang.TYPE.PCKG
import Lang.EXPR.PCKG
import Lang.STMT.PCKG
import SEQ_data
import PrimitiveConcepts.LC
import PrimitiveConcepts.Name
import SupportLib.MoreParserBasic

pKey:: String -> Parser Token String
pSpec:: Char -> Parser Token String
pKey  = UU.Scanner.pKey
pSpec = UU.Scanner.pSpec

type ParserType = Parser Token Type
type ParserType_wSize = Parser Token (Type,[Expr])
type ParserExpr = Parser Token Expr
type ParserStmt = Parser Token Stmt
type ParserSeqProg  = Parser Token SeqProg

p_SeqProg :: ParserType 
		-> ParserType_wSize 
		-> ParserType 
		-> ParserExpr 
		-> ParserStmt 
		-> ParserSeqProg

p_SeqProg 
	pParamType
	pDeclType
	pLCType
	pSpecExpr 
	pStmt
	= 
	semfun 
	<$  pKey "prog" <*> p_Name
	<*> (pParensIR . pListSep (pSpec ';') $ p_Param)
	<*> pListSep (pSpec ';') p_Decl
	<*> p_PreCond
	<*  pKeyAI "do" <*> pStmt
	<*> p_PostCond
	<*> pListSep (pSpec ';') p_Specification
	where
	semfun progname params decls prec body postc specs 
		=
		SeqProg progname params decls body specs_ lconsts
		
		where
		--
		-- Currently only consider 1 specification.
		-- The rest is ignored.
		--
		specs_ = take 1 (spec0 ++ (concat . map fst $ specs))
		spec0 = case (prec,postc) of
			(Just p, Just q)	-> [Spec p q]
			(Just p, Nothing)	-> [Spec p tt]
			(Nothing, Just q)	-> [Spec tt q]
			_			-> []
	
		lconsts = concat . map snd $ specs
		
	p_PreCond =  Just <$ pKeyXI "pre" <*> pSpecExpr
			<|> 
			pSucceed Nothing

	p_PostCond = Just <$ pKeyXI "post" <*> pSpecExpr 
			<|> 
			pSucceed Nothing


	p_Specification = 
		f1 <$> p_LogicConstList
		<|>
		f2 <$> p_prepost <*>  (p_LogicConstList <|> pSucceed [])
		where
		f1 consts = ([],consts)
		f2 prepost_spec consts = ([prepost_spec],consts)

	p_prepost = Spec 
			<$  pKeyXI "spec"
			-- currently specification name is ignored
			--
			<*  pVarid
			<*  pKey "pre"
			<*> pSpecExpr
			<*  pKeyAI "post"
			<*> pSpecExpr 

	--
	-- Currently only supports prefix lconst, 
	-- with no 2backend translation name
	--
	p_LogicConst = f
			<$> p_Names
			<*  pKeyAI "::"
			<*> pLCType

			where
			f names ty = map g names
			  where
			  g (Name pos name) = (pos,
					  UserDefinedLogicalConstant
						name
						ty
						Prefix
						name)

	p_LogicConstList = pKeyXI "lconst"
			    *> (concat <$> pList1Sep (pSpec ';') p_LogicConst)


	p_Param = f 
		<$> 
		(ReadOnly <$ pKeyXI "readonly" <|> pSucceed MayWrite)
		<*> p_Names
		<*  pKey "::"
		<*> (ByRef <$ pSpecXI '&' <|> pSucceed ByValue)
		<*> pParamType
		where
		f accessMode names passMode ty 
			= 
			Parameter passMode accessMode names ty 

	p_Decl = p_localvar <|> p_auxvar

	p_localvar = f
		<$> p_Names	
		<*  pKey "::" <*> pDeclType
		where
		f vars (ty,sizes) = Decl LocalVar vars ty sizes

	p_auxvar = f
		<$  pKeyXI "aux"
		<*> p_Names	
		<*  pKey "::" <*> pDeclType
		where
		f vars (ty,sizes) = Decl AuxVar vars ty sizes


p_Name :: Parser Token Name
p_Name = f <$> pVaridPos 
	where
	f (name,p) = Name p name

p_Names = pList1Sep (pSpec ',') p_Name
