-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //


module Lang.STMT.Common_Parser

where

import UU.Scanner hiding (pKey,pSpec)
import qualified UU.Scanner (pKey,pSpec)
import UU.Parsing 
import Lang.EXPR.Parsers
import Lang.EXPR.PCKG
import Lang.STMT.PCKG
import SupportLib.MoreParserBasic

--
--
-- import EXPR.Common_Parser (p_Ident, p_Identifier, p_ArrayRecordApp)

pKey:: String -> Parser Token String
pSpec:: Char -> Parser Token String
pKey  = UU.Scanner.pKey
pSpec = UU.Scanner.pSpec


type ParserExpr = Parser Token Expr
type ParserStmt = Parser Token Stmt

p_Skip = Skip <$  pKey "skip"

p_Asg p_rhs
	= 
	f  <$> pLHSAssign <*> pKeyPosAI ":=" <*> p_rhs
	where
	f lhs pos rhs = Asg pos lhs rhs

p_Seq p_stmt 
	= Seq 
	<$ pSpecXI '{' 
	<*> pList1Sep (pSpec ';') p_stmt 
	<* (pSpecAI '}' <|> pSpec ';' <* pSpecAI '}')

p_Loop p_simpleexpr p_expr p_body
	= 
	f0 <$> pInv 
	<*> (f0_1 <$> pMetric <*> pWhile 
	     <|> f0_2 <$> pWhile
	     <|> f0_2 <$> pForLoop)

	where
	f0 i (m,c,r) = r i m c  
	f0_1 (m,c) r = (m,c,r)
	f0_2 r       = (NOTHING,NOTHING,r)

	pInv  = JUST <$ pKeyXI "inv" <*> p_expr
		<|>
		pSucceed NOTHING

	pMetric = (f <$ pKeyXI "metric" <*> pCVAR <*> p_expr)
		<|>
		pSucceed (NOTHING,NOTHING) 
		where
		f c m = (JUST m, c)

	pCVAR = (JUST <$> p_Ident <* pKeyAI ":=")
		<|>
		pSucceed NOTHING

	pWhile = f1 <$> pKeyPosxI "while" 
			<*> p_simpleexpr
			<*  pKeyAI "do"
			<*> p_body

	f1 pos g body = (\i m c -> While pos i m c g body)

	pForLoop = f2 <$> pKeyPosxI "for"
			<*> p_Identifier
			<*> pKey "from"
			<*> p_simpleexpr 
			<*> pKeyAI "to"
			<*> p_simpleexpr 
			<*> pSpecAI ';'
			<*> p_simpleexpr 
			<*> pKeyAI "do"
			<*> p_body
	

	f2 pos ctr _ lb _ ub _ incr _ b 
		= 
		(\i m c -> ForLoop pos i m c lb ctr ub incr b)


p_Cond p_simpleexpr p_stmt = IfElse    
		<$> pKeyPosXI "if" 
		<*> p_simpleexpr
		<*  pKeyAI "then"  
		<*> p_stmt
		<*  pKeyAI "else" 
		<*> p_stmt



