-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //

module Lang.TYPE.Common_Parser

where
--
-- Define things that can be common to various variants 
-- of TYPE parsers. For example: combinators.
--

import UU.Parsing 
import UU.Scanner hiding (pKey,pSpec)
import qualified UU.Scanner (pKey,pSpec)
import TYPE_data 
import Lang.TYPE.Util
import SupportLib.MoreParserBasic

pKey:: String -> Parser Token String
pSpec:: Char -> Parser Token String
pKey  = UU.Scanner.pKey
pSpec = UU.Scanner.pSpec


--
-- A common parser for the array type
--
pArrayType pElemTy = lipat <$> pElemTy <*> pList p_brackets

	where
	p_brackets   = pSpecXI '[' <* pSpecAI ']'
	lipat e dims = foldr (const mk_ArrayTy) e dims

--
-- Common parser for function type
--
pFuncType pElemTy = pElemTy <??> (lipat <$> pList1 (pKeyXI "->" *> pElemTy))
	where
	lipat tyargs tyf = foldr1 mk_FunTy (tyf:tyargs)

--
-- A common parser for the list type
--
pListType pElemTy = mk_ListTy <$> pBracks pElemTy

-- 
-- A common parser for the record type
--
pRecordType pFieldTy = 
	TyRecord <$ pKeyXI "Record" <*> (pSpecAI '{' *> pFields <* pSpecAI '}')
	where
	pFields   = concat <$> pListSep pSemi pFields_
	pFields_  = f <$> pFieldNames <* pKeyAI "::" <*> pFieldTy
		where
		f fieldnames ty = map (\fn-> Field fn ty) fieldnames
	pFieldNames = pListSep pComma pConid

--
-- Atom types: tvar, primitive types, 
--             0-argument user-defined type 
--             or some top type in brackets
--
pAtomTy pTop = 
	(TyVar <$> pVarid) 	<|>
	pPrimType 		<|>
	(\tname-> TyCons tname []) <$> pConid	<|>	-- user defined type-constant
	pParens pTop
 
--
-- Parser for the primitive types
--
pPrimType =  
	mk_StringTy	<$ pKey stringTyCons 	<|>	
	mk_IntTy	<$ pKey intTyCons	 <|>	
	mk_BoolTy	<$ pKey boolTyCons	 <|>
	mk_CharTy	<$ pKey charTyCons		



