-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //
module Lang.TYPE.Util where

import TYPE_data
import List(nub)

voidTyCons	= "Void"
boolTyCons	= "Bool"
intTyCons	= "Int"
charTyCons	= "Char"
stringTyCons	= "String"
listTyCons	= "List"
arrayTyCons	= "Array"
funTyCons	= "Fun"
tableTyCons	= "Table"

boolTy		= TyCons boolTyCons []
intTy		= TyCons intTyCons []
stringTy	= TyCons stringTyCons []
charTy		= TyCons charTyCons []

is_BoolTy (TyCons tcons []) = tcons == boolTyCons
is_BoolTy _ = False

is_IntTy (TyCons tcons []) = tcons == intTyCons
is_IntTy _ = False

is_CharTy (TyCons tcons []) = tcons == charTyCons
is_CharTy _ = False

is_StringTy (TyCons tcons []) = tcons == stringTyCons
is_StringTy _ = False

is_BasicTy ty = foldr (||) False 
		. map (\f-> f ty) 
		$  [is_BoolTy
			,is_IntTy
			,is_CharTy
			,is_StringTy]

is_ListTy (TyCons tcons [_]) = tcons == listTyCons
is_ListTy _ = False

is_ArrayTy (TyCons tcons [_]) = tcons == arrayTyCons
is_ArrayTy _ = False

is_FunTy (TyCons tcons [_,_]) = tcons == funTyCons
is_FunTy _ = False

is_Fun2Ty (TyCons tcons1 [_, TyCons tcons2 [_,_]]) = (tcons1==funTyCons) && (tcons2==funTyCons)
is_Fun2Ty _ = False

is_RecordTy (TyRecord _) = True
is_RecordTy _ = False

is_TyVar (TyVar _) = True
is_TyVar _ = False

mk_VoidTy	= TyCons voidTyCons []
mk_BoolTy	= TyCons boolTyCons []
mk_IntTy 	= TyCons intTyCons  []
mk_CharTy	= TyCons charTyCons []
mk_StringTy	= TyCons stringTyCons []
mk_TyVar v	= TyVar v
mk_ListTy tye	= TyCons listTyCons   [tye]
mk_ArrayTy tye	= TyCons arrayTyCons  [tye]
mk_FunTy td tr	= TyCons funTyCons [td,tr]
infixr 5 ->>
(->>) = mk_FunTy

ty_b_b   = mk_BoolTy ->> mk_BoolTy
ty_b_b_b = mk_BoolTy ->> ty_b_b
ty_i_i_b = mk_IntTy ->> mk_IntTy ->> mk_BoolTy
ty_i_i_i = mk_IntTy ->> mk_IntTy ->> mk_IntTy


mk_RecordTy fields = TyRecord (map f fields)
	where
	f (fname,fty) = Field fname fty


dest_FunTy ty = case ty of
	TyCons tcons [tf,ta] -> if tcons == funTyCons then (tf,ta) else err 
	_ -> err
	where
	err = error "dest_FunTy on TYPE which is not a funTy"

dest_Fun2Ty ty = (td1,td2,tr)
	where
	(td1,tr_) = dest_FunTy ty
	(td2,tr)  = dest_FunTy tr_

dest_RecordTy (TyRecord fields) = map f fields
	where
	f (Field fn ty) = (fn,ty)
dest_RecordTy _ = error "dest_RecordTy on TYPE which is not a record type"

get_ListElTy ty = case ty of
	TyCons tcons [te] -> if tcons == listTyCons then te else err 
	_ -> err
	where
	err = error "get_ListElTy on TYPE which is not a listTy"

get_ArrayElTy ty = case ty of
	TyCons tcons [te] -> if tcons == arrayTyCons then te else err 
	_ -> err
	where
	err = error "get_ArrayElTy on TYPE which is not a arrayTy"

get_ArrayBaseTy ty = case ty of
	TyCons tcons [te] -> if tcons == arrayTyCons 
				then get_ArrayBaseTy te
				else ty
	_  ->  ty


instance Eq FieldType where
	Field n1 ty1 == Field n2 ty2 	= (n1==n2) && (ty1==ty2)

instance Eq Type where
	TyVar v1 == TyVar v2 	= v1==v2
	TyCons tc1 ts1 == TyCons tc2 ts2	= (tc1==tc2) && (ts1==ts2)
	TyRecord tfs1 == TyRecord tfs2		= tfs1 == tfs2
	_ == _			= False
	




