-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //
--
-- The main of litemech
--
module Main 

where

import Main.ParserAPI
import Main.LogicAPI
import Main.ToHOLAPI
import Main.HelpAPI
import SupportLib.MoreIO
import Main.Option

--
-- importing other standard Haskell utilities
--
import Maybe
import List
import System(getArgs)
 
-- ============================================================
-- 
-- Top level interface: main and reduce
--
-- ============================================================

--
-- main for the batch-mode exe
--
main :: IO ()
main = do {args <- getArgs ; main_ args }

--
-- The top level reduction function
--
reduce :: String -> IO()
reduce arg = main_ (words arg) 

main_ :: [String] -> IO()
main_ args
   	= 
	
	if (null options && noSourceFile) || ShowHelp `elem` options

	then putPP pp_mainOptionsHelp

	else if ShowBanner `elem` options then putPP pp_mechBanner

	else if ShowCredit `elem` options then putPP pp_credits

	else if noSourceFile then writeStr_ logfile "\nNo source is specified." 

	else reducePrim options sourcefile

	where

	(fname_,options)	= getOptions args 

	noSourceFile		= null fname_

	sourcefile		= fname_ !! 0

	logfile			= sourcefile ++ ".log"


-- ===============================================================
--
-- the main reducer application (called by top level reduce)
-- 
-- ===============================================================

--
-- NOTE: this function relies a lot on laziness so that its top
-- level computation flow appears to be simple.
--

reducePrim :: [MainOption] -> String -> IO()
reducePrim options source 
	=
	do
  	{ --
	  -- placing all the engines... 
	  -- uhm.. lazy effect here mind you.
	  --
	  (ptree,pp_parse_errors) 		<- parser parsingMode source
	; (syntax_errors,ppse)	 		<- syc parsingMode ptree
	; (ptree',ty_errors,ppte)		<- tyc ptree
	; (vcs,pp_vcs)				<- vcgen reduceOption ptree'
	; pp_holgoal				<- toHOL vcs

	--
	-- now for printing the outputs
	--
	; writeStr_ logfile ("\nSource file: " ++ source ++ "\n")
	; if isJust pp_parse_errors
	  then writePP_ logfile (fromJust pp_parse_errors)
	  else if not (null syntax_errors)
	  then writePP_ logfile ppse
	  else if not (null ty_errors)
	  then writePP_ logfile ppte
	  else 
	  do { putStr "No syntax and type error found.\n"
	     ; if ToggleOffReduce `elem` options
	       then putStr "No reduction is done at your request." 
	       else 
		do {putStr (showReductionOption reduceOption ++ "\n")
		   ; if ToggleOnEO `elem` options
	       		then writePP_ outfile pp_vcs
	       		else writePP False outfile pp_vcs
		   ; writePPW False houtfile pp_holgoal
		   }
	     }
	; putStr "\nDone."
	}

	where

	logfile			= source ++ ".log"

	reduceOption		= getReductionOption options

	parsingMode		= if ToggleOnQL `elem` options
				  then QL
				  else BASIC

	ext_name		= fext_name reduceOption

	outfile			= case getOutputBaseFileName options of
				  Just fn	-> fn ++ ".out" ++ ext_name
				  _		-> source ++ ".out" ++ ext_name

	houtfile		= case getOutputBaseFileName options of
				  Just fn	-> fn ++ ".hout" ++ ext_name
				  _		-> source ++ ".hout" ++ ext_name



