-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //
module Main.Option(
	MainOption(..)
	,getOptions
	,pp_mainOptionsHelp
	,getReductionOption
	,showReductionOption
	,getOutputBaseFileName
	,fext_name)

where

import GetOpt
import List
import Engine.VCG.PCKG
import Configuration.Face
import UserInteraction.Screen
--
-- x-MECH LITE options
--
data MainOption =
	  SelectRO ReduceOption
	| ToggleOnQL
	| ToggleOnEO
	| ToggleOffReduce
	| CustomOutFile  String
	| ShowPtree
	| ShowHelp
	| ShowBanner
	| ShowCredit
	deriving (Eq,Show)

--
-- Short textual descriptions of the options
--
mainOption_desc option = case option 
  of
  SelectRO RO_ATotal 	-> "idealized reduction implying total correctness (default)"
  SelectRO RO_APartial  -> "idealized reduction implying partial correctness"
  SelectRO RO_T		-> "partial reduction implying termination"
  SelectRO RO_AOC	-> "partial reduction implying array-overflow safe"
  SelectRO RO_DZC	-> "partial reduction implying division-by-zero safe"
  SelectRO RO_All	-> "combined reduction"
  ToggleOnQL		-> "turn-on the Query Language extension"
  ToggleOnEO		-> "echo reduction result"
  ToggleOffReduce	-> "surpress reduction (syntax-check only)"
  CustomOutFile  _ 	-> "specifiy the base-name of the output files"
  ShowHelp		-> "help on available options"
  ShowBanner		-> "show our banner"
  ShowCredit		-> "show the credits"	
  ShowPtree		-> "only echo the parse tree of the program (developer option)"
  _			-> "unknown option"

--
-- Listing all options for getOpt
--
mainOptions :: [OptDescr MainOption]
mainOptions =
	let
	sel_ATotal 	= SelectRO RO_ATotal
	sel_APartial	= SelectRO RO_APartial
	sel_T		= SelectRO RO_T	
	sel_AOC		= SelectRO RO_AOC	
	sel_DZC		= SelectRO RO_DZC
	sel_All		= SelectRO RO_All

	in
  	[ Option 
		[] 
		["total"]
		(NoArg sel_ATotal)
		(mainOption_desc sel_ATotal)
	 ,Option
		[]
		["partial"]
		(NoArg sel_APartial)
		(mainOption_desc sel_APartial)
	 ,Option
		[]
		["tc"]
		(NoArg sel_T)
		(mainOption_desc sel_T)    
	 ,Option
		[]
		["dzc"]
		(NoArg sel_DZC)
		(mainOption_desc sel_DZC)    
	 ,Option
		[]
		["aoc"]
		(NoArg sel_AOC)
		(mainOption_desc sel_AOC)    
	 ,Option
		['a']
		[]
		(NoArg sel_All)
		(mainOption_desc sel_All)    
	 ,Option
		['q']
		[]
		(NoArg ToggleOnQL)
		(mainOption_desc ToggleOnQL)   
	 ,Option
		['e']
		[]
		(NoArg ToggleOnEO)
		(mainOption_desc ToggleOnEO)   
	 ,Option
		['s']
		[]
		(NoArg ToggleOffReduce)
		(mainOption_desc ToggleOffReduce)
	 ,Option
		['o']
		[]
		(ReqArg (\basename-> CustomOutFile basename) "filename")
		(mainOption_desc (CustomOutFile ""))
	 ,Option
		['h']
		[]
		(NoArg ShowHelp)
		(mainOption_desc ShowHelp)
	 ,Option
		[]
		["banner"]
		(NoArg ShowBanner)
		(mainOption_desc ShowBanner)
	 ,Option
		[]
		["credit"]
		(NoArg ShowCredit)
		(mainOption_desc ShowCredit)
	 ,Option
		[]
		["echoparsetree","ep"]
		(NoArg ShowPtree)
		(mainOption_desc ShowPtree)    
	]

    
getOptions :: [String] -> ([String],[MainOption])
getOptions args 
	= 
	if not (null errors)
   	then error (concat errors)
	else (fnames,options ) 
	where
	(options,non_options,errors) = getOpt Permute mainOptions args

	--
	-- only take the first non_options as source file name
	-- the rest is ignores!
	--
	fnames = take 1 non_options


--
-- pp to list all available options (used for user-help)
--
pp_mainOptionsHelp
	= 
	  pp_help
	. map reformatLine
	. lines
	. usageInfo "Help on available options:\n" 

	$ mainOptions

	where
	reformatLine row = tab ++ dropWhile (== ' ') row
	tab = replicate std_indentWidth ' '

--
-- Infer what the reduction options are
--
getReductionOption :: [MainOption] -> ReduceOption
getReductionOption options 
	= 
	case filter allowed options of

	[]				-> RO_ATotal --default
	(SelectRO reduceoption: _ )	-> reduceoption
	

	where

	allowed (SelectRO _) = True
	allowed _            = False


--
-- String to show to the user what are the inferred
-- reduction options
--
showReductionOption option = "Reduction mode: " ++ showOption_ option
	where
	showOption_ option = case option of

		RO_ATotal	-> "total."
		RO_APartial	-> "partial."
		RO_T		-> "tc."
		RO_AOC		-> "aoc."
		RO_DZC		-> "dzc."
		RO_All 		-> "all."

--
-- Infer the base-name for the output file
--
getOutputBaseFileName :: [MainOption] -> Maybe String
getOutputBaseFileName options = f . find p $ options
	where
	p (CustomOutFile _)	= True
	p _			= False
	f (Just (CustomOutFile fn))	= Just fn
	f _				= Nothing


--
-- File extension names
--
fext_name :: ReduceOption -> String
fext_name opt =
	case opt of
	RO_ATotal	-> ".std"
	RO_APartial	-> ".partial"
	RO_T		-> ".tc"
	RO_AOC		-> ".aoc"	
	RO_DZC		-> ".dzc"
	_		-> ".all"
