-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //
--
-- For representing the specification of (User Defined) 
-- Logical Constants
--
module PrimitiveConcepts.LC(
	UserDefinedLogicalConstant(..)
	,Fixity(..)
	,get_prefixes
	,get_infixes
	,get_quanties
	,order_infixes
	,isRighty
	,getPriority
	)

where

import UU.Pretty
import Lang.TYPE.PCKG

--
-- That is ... suport for user defined prefixes and infixes for
-- x-MECH LITE (nothing fancy like syntax macro here).
--

--
-- A data structure to specify a user defined logical constant.
--

data UserDefinedLogicalConstant 
	=
	UserDefinedLogicalConstant
	 { lc_name	:: String		--constant's name
	 , lc_type	:: Type			--constant's type 
	 , lc_fixity	:: Fixity
	 , lc_translation :: String }

instance Show UserDefinedLogicalConstant where
  showsPrec _ (UserDefinedLogicalConstant name ty fix trans) s
	=
	"\n" ++ name ++ " :: " ++ show ty
	++ " " 
	++ show fix
	++ " ("
	++ trans ++ ")"
	++ s

instance PP UserDefinedLogicalConstant where
  pp (UserDefinedLogicalConstant name ty fix trans)
	=
	text name >#< text "::" >#< pp ty 
	>#< (pp_parens . text . show $ fix)
	>#< pp_parens (text trans)
--
-- Note: it is possible that the defined constant may be overloaded
-- (e.g. equality on lists). We will rely on the back-end prover to
-- resolve the overloading. 
--

--
-- Associativity
--
data Fixity =
	  NonAssoc	Int		--non-associative infix
	| Lefty		Int		--left infix with specified priority
	| Righty	Int		--right infix
	| Prefix
	| Quanty
	deriving (Show,Eq)

isInfix = p . lc_fixity
	where
	p (Lefty _)  = True
	p (Righty _) = True
	p (NonAssoc _) = True
	p _ = False

isRighty = p . lc_fixity
	where
	p (Righty _) = True
	p _ = False

isPrefix  = (==Prefix) . lc_fixity
isQuanty = (==Quanty) . lc_fixity

get_infixes  = filter isInfix
get_prefixes = filter isPrefix
get_quanties = filter isQuanty

--
-- Assuming an infix
--
getPriority = f . lc_fixity
	where
	f (Lefty i)    = i
	f (Righty i)   = i
	f (NonAssoc i) = i

--
-- Group and order list of infixes according to their priority
-- and associativity. Infixes with the same priority and 
-- associativity are grouped together, then the group is sorted.
-- 

order_infixes user_infixes = map snd . sortir . group $ user_infixes

	where

	group [] = []
	group (x:s) = x' : group s'
	  where
	  x' = (lc_fixity x, x : filter (equalFix x) s)
	  s' = filter (not . equalFix x) s

	equalFix x y = lc_fixity x == lc_fixity y

	prio (NonAssoc n) = n
	prio (Lefty n)    = n
	prio (Righty n)   = n

	sortir []    = []
	sortir (x:s) = insert x (sortir s)
	insert x [] = [x]
	insert x s@(y:t) 
		| prio (fst x) <= prio (fst y)	= x:s
		| otherwise			= y : insert x t
