-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //

--
-- Provide alternate basic parsers (at the token level)
-- Needed for finer tuning of correction cost.
--
module SupportLib.MoreParserBasic

where

import UU.Scanner.Position(Pos)
import UU.Scanner.GenTokenParser(pCostReserved,pCostValToken)
import UU.Scanner(pSpec,Token,EnumValToken(..))
import UU.Parsing

--
-- Keyword and spec-sym parsers like pKey and pSpec, 
-- but set the insertion cost at maximum (XI variants),
-- minimum (AI variants), and middle (xI variants).
--
pKeyXI :: String -> Parser Token String
pKeyXI keyword  =  keyword <$ pCostReserved 1000 keyword

pSpecXI :: Char -> Parser Token String
pSpecXI s       =  [s] <$ pCostReserved 1000 [s]

pKeyPosXI :: String -> Parser Token Pos
pKeyPosXI keyword  =  pCostReserved 1000 keyword

pSpecPosXI :: Char -> Parser Token Pos
pSpecPosXI s       =  pCostReserved 1000 [s]

pKeyAI :: String -> Parser Token String
pKeyAI keyword  =  keyword <$ pCostReserved 5 keyword

pSpecAI :: Char -> Parser Token String
pSpecAI s       =  [s] <$ pCostReserved 5 [s]

pKeyPosAI :: String -> Parser Token Pos
pKeyPosAI keyword  =  pCostReserved 5 keyword

pSpecPosAI :: Char -> Parser Token Pos
pSpecPosAI s       =  pCostReserved 5 [s]

pKeyxI :: String -> Parser Token String
pKeyxI keyword  =  keyword <$ pCostReserved 10 keyword

pSpecxI :: Char -> Parser Token String
pSpecxI s       =  [s] <$ pCostReserved 10 [s]

pKeyPosxI :: String -> Parser Token Pos
pKeyPosxI keyword  =  pCostReserved 10 keyword

pSpecPosxI :: Char -> Parser Token Pos
pSpecPosxI s       =  pCostReserved 10 [s]

--
-- parsers for varid and conid, with maximum insertion cost
--

pVaridPosXI,pConidPosXI :: Parser Token (String,Pos)
pVaridPosXI   =   pCostValToken 1000 TkVarid     "<identifier>" 
pConidPosXI   =   pCostValToken 1000 TkConid     "<Identifier>" 

pVaridXI, pConidXI :: Parser Token String
pVaridXI      =   fst <$> pCostValToken 1000 TkVarid     "<identifier>" 
pConidXI      =   fst <$> pCostValToken 1000 TkConid     "<Identifier>" 

--
-- Parens, brackets, and curly packing parsers, but set such that
-- inserting a closing )/}/] is cheap.
--
pParensIR, pBracksIR, pCurlyIR :: Parser Token a -> Parser Token a

pParensIR  = pPacked (pSpecXI '(') (pCostReserved 5 ")")
pBracksIR  = pPacked (pSpecXI '[') (pCostReserved 5 "]")
pCurlyIR   = pPacked (pSpecXI '{') (pCostReserved 5 "}")

--
-- integer parser, with more expensive insert cost
--
pIntegerPos_i :: Parser Token (String,Pos)
pIntegerPos_i = pCostValToken 10 TkInteger10 "0"

--
-- Char and string parser, with maximum insert cost
--
pCharPosXI,pStringPosXI :: Parser Token (String,Pos)
pCharPosXI   = pCostValToken 1000 TkChar   "\NUL"
pStringPosXI = pCostValToken 1000 TkString ""
