-- //===========================================================================
-- //
-- // AUTHOR(S): Wishnu Prasetya, Ade Azurat
-- // PACKAGE  : (c)xMECH, class LITE, version alpha 1.3.2
-- //
-- //===========================================================================
-- //

--
-- Provide some top level interface to call a parser
--
module SupportLib.MoreParserTopInterface(
	 ShowErrorMode(..)
	,pp_ParseErrors
	,parseString
	,parseStr
	,parseFile
	)

where

import Configuration.Face
import UU.Parsing
import UU.Pretty
import Maybe
import List

data ShowErrorMode =
	  ShowErrorMode_ALL
	| ShowErrorMode_SOME Int
	| ShowErrorMode_NONE

parse_ parser inp = (result,messages)
	where
	steps         = parse parser inp
	Pair result _ = evalSteps steps
	messages      = getMsgs steps

--
-- string parser
--
parseString stringScanner parser inputString
	=
	(return . parse_ parser . stringScanner) inputString 

--
-- Simple string parser. No correction. Very simple error mesage.
-- Fails with error.
--
parseStr scanner parser input = fromJust m  --casting back from monad
	where
	m = do
		{ (result,messages) <- parseString scanner parser input 
		; if null messages
		  then return result
		  else error (mk_error_txt messages)
		}
	mk_error_txt ms = "\n" ++ disp (ppErrors ms) std_PageWidth ""
	ppErrors        = pp_ParseErrors (ShowErrorMode_SOME 1)

--
-- file parser
--
parseFile fileScanner parser file
	=
	do 
	{ toks <- fileScanner file ;
	  return (parse_ parser toks)  }


pp_ParseError (Msg expecting position action)  
          =  let pos = case position of
                           Nothing -> "at end of file"
                           Just s  -> case action of 
                                Insert _ -> "before " ++ show s
                                Delete t -> "at " ++ show t  
             in 
		vlist
		[ text "Parse error:" >#< text pos 		,
		  text "Expecting  :" >#< show expecting	,
                  text "Repaired by:" >#< show action		]

pp_ParseErrors_ []    = text "No parse error."
pp_ParseErrors_ [e]   = pp_ParseError e
pp_ParseErrors_ (e:s) = pp_ParseError e
			>-< text hline
			>-< pp_ParseErrors_ s

hline = replicate std_PageWidth '-'

pp_ParseErrors mode messages
	= 
	if null messages 
	then text "No parse error."
	else
		case mode of
		ShowErrorMode_SOME n    -> pp_ParseErrors_ (take n messages)
		ShowErrorMode_NONE	-> text "Parse error ... somewhere."
		_			-> pp_ParseErrors_ messages



